package org.processmining.plugins.declareminer;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.declareminer.apriori.FindItemSets;
import org.processmining.plugins.declareminer.enumtypes.DeclareTemplate;
import org.processmining.plugins.declareminer.enumtypes.FrequentItemSetType;
import org.processmining.plugins.declareminer.templates.Absence2Info;
import org.processmining.plugins.declareminer.templates.AbsenceInfo;
import org.processmining.plugins.declareminer.templates.ChoiceInfo;
import org.processmining.plugins.declareminer.templates.CoexistenceInfo;
import org.processmining.plugins.declareminer.templates.Exactly1Info;
import org.processmining.plugins.declareminer.templates.ExclusiveChoiceInfo;
import org.processmining.plugins.declareminer.templates.ExistenceInfo;
import org.processmining.plugins.declareminer.templates.InitInfo;
import org.processmining.plugins.declareminer.templates.NegativeRelationInfo;
import org.processmining.plugins.declareminer.templates.NotCoexistenceInfo;
import org.processmining.plugins.declareminer.templates.PrecedenceInfo;
import org.processmining.plugins.declareminer.templates.ResponseInfo;
import org.processmining.plugins.declareminer.templates.SuccessionInfo;
import org.processmining.plugins.declareminer.templates.TemplateInfo;
import org.processmining.plugins.declareminer.visualizing.ConstraintTemplate;
import org.processmining.plugins.declareminer.visualizing.DeclareMinerOutput;

public class Pruner {

	/**
	 * @param cpir2 
	 * @param log 
	 * @param posfrequentItemSetSupportMap 
	 * @param events 
	 * @param supp 
	 * @param args
	 */

	private PrintWriter pw;
	private UIPluginContext context;
	private XLog log;
	private DeclareMinerInput input;

	public Pruner(UIPluginContext context, XLog log, DeclareMinerInput input, PrintWriter printWriter){
		this.context = context;
		this.log = log;
		this.pw = printWriter;
		this.input = input;
	}

	private boolean optimized = false;
	private Map<DeclareTemplate, Vector<MetricsValues>> metricsValuesPerTemplate;
	private FindItemSets f;

	public DeclareMinerOutput prune(boolean hier, boolean trans,List<String> activityNameList,  Map<FrequentItemSetType, Map<Set<String>, Float>> frequentItemSetTypeFrequentItemSetSupportMap, Map<DeclareTemplate, List<List<String>>> declareTemplateCandidateDispositionsMap, Hashtable<?, ?> aprioriSupportValues, boolean verbose, DeclareMinerInput input){
		if(pw!=null){
			pw.println("START PRUNING");
		}
	
		Watch pruneWatch = new Watch();
		pruneWatch.start();
		float support = input.getMinSupport()/100.f;
		float alpha = input.getAlpha()/100.f;
		f = new FindItemSets(activityNameList,log, input);

		Watch pruneLocalWatch = new Watch();
		pruneLocalWatch.start();
		HashMap<String, MetricsValues> metricsValues4precedence = null;
		HashMap<String, MetricsValues> metricsValues4response = null;

		metricsValuesPerTemplate = new HashMap<DeclareTemplate, Vector<MetricsValues>>();


		TemplateInfo templateInfo = null;
		Set<DeclareTemplate> selectedTemplateList = input.getSelectedDeclareTemplateSet();
		for(DeclareTemplate selectedDeclareTemplate : selectedTemplateList){
			startLogging(selectedDeclareTemplate, pruneLocalWatch);
			Vector<MetricsValues> metricsValues = new Vector<MetricsValues>();
			switch(selectedDeclareTemplate){
				case Succession:
				case Alternate_Succession:
				case Chain_Succession:
					templateInfo = new SuccessionInfo();
					break;
				case Choice:
					templateInfo = new ChoiceInfo();
					break;
				case Exclusive_Choice:
					templateInfo = new ExclusiveChoiceInfo();
					break;
				case Existence:
				case Existence2:
				case Existence3:
					templateInfo = new ExistenceInfo();
					break;
				case Init:
					templateInfo = new InitInfo();
					break;
				case Absence:
					templateInfo = new AbsenceInfo();
					break;
				case Absence2:
				case Absence3:
					templateInfo = new Absence2Info();
					break;
				case Exactly1:
				case Exactly2:
					templateInfo = new Exactly1Info();
					break;
				case Precedence:
				case Alternate_Precedence:
				case Chain_Precedence:
					templateInfo = new PrecedenceInfo();
					break;
				case Responded_Existence:
				case Response:
				case Alternate_Response:
				case Chain_Response:
					templateInfo = new ResponseInfo();
					break;
				case CoExistence:
					templateInfo = new CoexistenceInfo();
					break;
				case Not_CoExistence:
					templateInfo = new NotCoexistenceInfo();
					break;
				case Not_Succession:
				case Not_Chain_Succession:
					templateInfo = new NegativeRelationInfo();
					break;	
			}
			templateInfo.setMetricsValues4precedence(metricsValues4precedence);
			templateInfo.setMetricsValues4response(metricsValues4response);
			metricsValues = templateInfo.getMetrics(input,frequentItemSetTypeFrequentItemSetSupportMap,declareTemplateCandidateDispositionsMap, alpha, support, log, pw, selectedDeclareTemplate, context, verbose, f);
			metricsValues4precedence = templateInfo.getMetricsValues4precedence();
			metricsValues4response = templateInfo.getMetricsValues4response();
			metricsValuesPerTemplate.put(selectedDeclareTemplate, metricsValues);
			endLogging(selectedDeclareTemplate, pruneLocalWatch,metricsValues.size(), templateInfo.getNumberOfDiscoveredConstraints());

		}
		if(pw!=null){
			pw.println("END PRUNING - time: "+pruneWatch.msecs()+" msecs");
			pw.flush();
		}

		for(DeclareTemplate template : metricsValuesPerTemplate.keySet()){
			Vector<MetricsValues> toRemove = new Vector<MetricsValues>();
			for(MetricsValues values : metricsValuesPerTemplate.get(template)){
				if(values.getSupportRule()<support){
					toRemove.add(values);
				}
			}
			metricsValuesPerTemplate.get(template).removeAll(toRemove);
		}

		if(pw!=null){
			pw.println("START TIME MODEL GENERATION");
		}
		Watch modelGenerationWatch = new Watch();
		modelGenerationWatch.start();
		DeclareModelGenerator dmg = new DeclareModelGenerator();
		DeclareMinerOutput output = dmg.createModel(activityNameList,hier, trans,metricsValuesPerTemplate,log,input, new Vector<String>(), new Vector<String>(), new Vector<String>());
		//output = DeclareModelGenerator.createModel(output, 0, 0, 0, 0);
		DeclareModelGenerator.layout(output.getModel().getView(), output.getModel().getModel());
		output.setMinSupport(support);
		output.setExtend(false);
		output.setAlpha((int)alpha);
		if(pw!=null){
			pw.println("END TIME MODEL GENERATION - time: "+modelGenerationWatch.msecs()+" msecs");
			pw.println("START TIME INFORMATION EVALUATION");
		}
		return output;
	}





	private void endLogging(DeclareTemplate declareTemplate, Watch pruneLocalWatch, int numberOfCandidates, int numberOfDiscoveredConstraints) {
		if(pw!=null){
			pw.println(" ");
			pw.println("# time for pruning "+ declareTemplate.toString()+" = "+pruneLocalWatch.msecs()+" msecs");
			pw.println("# number of candidates "+declareTemplate.toString()+" constraints "+ numberOfCandidates);
			pw.println("# number of discovered "+declareTemplate.toString() +" constraints "+ numberOfDiscoveredConstraints);
			pw.println(" ");
		}

	}



	private void startLogging(DeclareTemplate declareTemplate, Watch pruneLocalWatch) {
		if(pw!=null){
			ConstraintTemplate constraintTemplate = input.getDeclareTemplateConstraintTemplateMap().get(declareTemplate);
			pw.println(" ");
			pw.println("******************************");
			pw.println("start pruning "+ constraintTemplate.getName());
			pw.println(constraintTemplate.getName()+ " rule: "+constraintTemplate.getText());
			pw.println(constraintTemplate.getName()+ " description: "+constraintTemplate.getDescription());
			pw.println("******************************");
			pw.println(" ");
			pruneLocalWatch.start();
		}
	}


	protected long getMaxTimeDistance(XLog log){
		return 0;
	}

	protected long getMinTimeDistance(XLog log){
		return 0;
	}

	protected long getAvgTimeDistance(XLog log){
		return 0;
	}

	public boolean isOptimized() {
		return optimized;
	}


	public void setOptimized(boolean optimized) {
		this.optimized = optimized;
	}


	public Map<DeclareTemplate, Vector<MetricsValues>> getMetrVectors() {
		return metricsValuesPerTemplate;
	}


	public void setMetrVectors(Map<DeclareTemplate, Vector<MetricsValues>> metrVectors) {
		this.metricsValuesPerTemplate = metrVectors;
	}

	public FindItemSets getF() {
		return f;
	}


	public void setF(FindItemSets f) {
		this.f = f;
	}


}


