package org.processmining.plugins.declareminer.ui;

import java.util.Map;
import java.util.Set;

import javax.swing.JFrame;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.declareminer.DeclareMinerInput;
import org.processmining.plugins.declareminer.enumtypes.AprioriKnowledgeBasedCriteria;
import org.processmining.plugins.declareminer.enumtypes.DataDeclareTemplate;
import org.processmining.plugins.declareminer.enumtypes.DeclarePerspective;
import org.processmining.plugins.declareminer.enumtypes.DeclareProMInput;
import org.processmining.plugins.declareminer.enumtypes.DeclareTemplate;
import org.processmining.plugins.declareminer.enumtypes.MapTemplateConfiguration;
import org.processmining.plugins.declareminer.listener.DeclareMinerSettingsListener;
import org.processmining.plugins.declareminer.swingx.ErrorDialog;
import org.processmining.plugins.declareminer.visualizing.ConstraintTemplate;


public class DeclareMinerConfigurationUI implements DeclareMinerSettingsListener {
	UIPluginContext context;
	DeclareProMInput declareProMInput;
	
	int noSteps;
	int currentStep;
	private myStep[] mySteps;
	
	private int mapTemplateConfigurationStep;
	private int templateConfigurationStep;
	private int aprioriConfigurationStep;
	private int declarePerspectiveConfigurationStep;
	
	DeclareMinerInput input;
	
	public DeclareMinerConfigurationUI(UIPluginContext context, DeclareProMInput declareProMInput){
		this.context = context;
		this.declareProMInput = declareProMInput;
		
		InteractionResult result = InteractionResult.NEXT;
		input = new DeclareMinerInput();
		
		noSteps = 0;
		mapTemplateConfigurationStep = noSteps++;
		templateConfigurationStep = noSteps++;
		aprioriConfigurationStep = noSteps++;
		declarePerspectiveConfigurationStep = noSteps++;
		
		mySteps = new myStep[noSteps];
		
		mySteps[mapTemplateConfigurationStep] = new MapTemplateActivityConfigurationStep();
		mySteps[mapTemplateConfigurationStep].setListener(this);
		
		mySteps[templateConfigurationStep] = new TemplateConfigurationStep();
		mySteps[templateConfigurationStep].setListener(this);
		
		mySteps[aprioriConfigurationStep] = new AprioriConfigurationStep();
		mySteps[aprioriConfigurationStep].setListener(this);
		
		mySteps[declarePerspectiveConfigurationStep] = new DeclarePerspectiveConfigurationStep();
		mySteps[declarePerspectiveConfigurationStep].setListener(this);
		
		if(declareProMInput == DeclareProMInput.Log_and_Model)
			currentStep = 0;
		else
			currentStep = 1;
		
		while (true) {
			if (currentStep < 0) {
				currentStep = 0;
			}
			if (currentStep >= noSteps) {
				currentStep = noSteps - 1;
			}
			
			if(declareProMInput == DeclareProMInput.Log_and_Model)
				result = context.showWizard("Declare Maps Miner Plugin", currentStep == 0, currentStep == noSteps - 1, mySteps[currentStep]);
			else
				result = context.showWizard("Declare Maps Miner Plugin", currentStep == 1, currentStep == noSteps - 1, mySteps[currentStep]);
		
			
			switch (result) {
				case NEXT:
					if(declareProMInput == DeclareProMInput.Log_and_Model && currentStep == mapTemplateConfigurationStep)
						go(2);
					else
						go(1);
					break;
				case PREV:
					if(declareProMInput == DeclareProMInput.Log_and_Model && currentStep == aprioriConfigurationStep)
						go(-2);
					else
						go(-1);
					break;
				case FINISHED:
					readSettings();
					//validating settings
					if(!validateSettings()){
						if(input.getSelectedDeclareTemplateSet().size() == 0){
							ErrorDialog.showErrorDialog(new JFrame(), "Choose atleast one template");
							currentStep = templateConfigurationStep;
							go(0);
							break;
						}
						if(input.getDeclarePerspectiveSet().size() == 0){
							ErrorDialog.showErrorDialog(new JFrame(), "Choose atleast one perspective");
							currentStep = declarePerspectiveConfigurationStep;
							go(0);
							break;
						}
					}
					else{
						return;
					}
					break;
				case CANCEL:
					input.setEmpty(true);
					return;
				default:
					return;
			}
		}
	}

	private int go(int direction) {
		currentStep += direction;
		if (currentStep >= 0 && currentStep < noSteps) {
			if (mySteps[currentStep].precondition()) {
				return currentStep;
			} else {
				return go(direction);
			}
		}
		return currentStep;
	}
	
	private void readSettings(){
		for(int currentStep = 0; currentStep < noSteps; currentStep++){
			mySteps[currentStep].readSettings();
		}
	}

	private boolean validateSettings(){
		if(declareProMInput == DeclareProMInput.Log_Only && input.getSelectedDeclareTemplateSet().size()==0)
			return false;
		if(input.getDeclarePerspectiveSet().size()==0)
			return false;
		return true;
	}
	
	public void setSelectedDeclareTemplateSet(Set<DeclareTemplate> selectedDeclareTemplateSet) {
		System.out.println("Setting delcare template set");
		input.setSelectedDeclareTemplateSet(selectedDeclareTemplateSet);		
	}

	public void setMinSupport(int minSupport) {
		System.out.println("Setting min support");
		input.setMinSupport(minSupport);
	}

	public void setAlpha(int alpha) {
		System.out.println("Setting alpha");
		input.setAlpha(alpha);
	}

	public void setDeclarePerspectiveSet(Set<DeclarePerspective> declarePerspectiveSet) {
		System.out.println("Setting delcare perspective");
		input.setDeclarePerspectiveSet(declarePerspectiveSet);
	}

	public DeclareMinerInput getInput() {
		return input;
	}

	public void setDeclareTemplateConstraintTemplateMap(Map<DeclareTemplate, ConstraintTemplate> declareTemplateConstraintTemplateMap) {
		input.setDeclareTemplateConstraintTemplateMap(declareTemplateConstraintTemplateMap);
	}

	public void setMapTemplateConfiguration(MapTemplateConfiguration mapTemplateConfiguration) {
		input.setMapTemplateConfiguration(mapTemplateConfiguration);
	}

	public void setAprioriKnowledgeBasedCriteria(Set<AprioriKnowledgeBasedCriteria> aprioriKnowledgeBasedCriteriaSet) {
		input.setAprioriKnowledgeBasedCriteriaSet(aprioriKnowledgeBasedCriteriaSet);
	}

	public void setAprioriKnowledgeConceptFileName(String aprioriKnowledgeConceptFileName) {
		input.setAprioriKnowledgeConceptFileName(aprioriKnowledgeConceptFileName);
	}

	public void setDataDeclareTemplateConstraintTemplateMap(
			Map<DataDeclareTemplate, ConstraintTemplate> declareTemplateConstraintTemplateMap) {
		// TODO Auto-generated method stub
		
	}

	public void setSelectedDataDeclareTemplateSet(Set<DataDeclareTemplate> selectedDeclareTemplateSet) {
		// TODO Auto-generated method stub
		
	}
}
