/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.declareminer.visualizing;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.plugins.declareminer.DeclareMinerInput;
import org.processmining.plugins.declareminer.DeclareModelGenerator;
import org.processmining.plugins.declareminer.enumtypes.DeclareTemplate;
import org.processmining.plugins.declareminer.gui.ListRendererResult;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.organic.JGraphOrganicLayout;

/**
 * @author Fabrizio M. Maggi
 * 
 */
public class SlickerOpenDeclareModel extends JPanel implements MouseListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 215385982744090270L;
	protected DGraph graph;
	private DeclareMinerOutput decViz;
	private DeclareMinerOutput vizMod;
	private boolean extend;
	private NiceSlider nofEventsSliderSupp;
	private NiceSlider nofEventsSliderConfidence;
	private NiceSlider nofEventsSliderInterestFactor;
	private NiceSlider nofEventsSliderCPIR; 
	private final JList abstrList = new JList();
	private final JList eventsList = new JList();
	private final JList eventsTypList = new JList();
	private Vector templatesToRemove = new Vector();
	private Vector etToRemove = new Vector();
	private String sortingMetric = "support";
	private Vector eventsToRemove = new Vector();
	protected Hashtable selectedTemplates = new Hashtable();
	protected Hashtable selectedEvents = new Hashtable();
	protected Hashtable selectedET = new Hashtable();
	protected Vector temAdded = new Vector();
	protected Vector eveAdded = new Vector();
	protected Vector etAdded = new Vector();
	protected ProMSplitPane mainPanel;
	protected Vector eventTypes = new Vector();
	private DeclareMinerInput input;
	private XLog log;
	Vector precs = new Vector();
	Vector inits = new Vector();
	JPanel leftPanel;
	ProMSplitPane rightPanel;
	Vector responds = new Vector();

	@Override
	public void mouseClicked(final MouseEvent arg0) {
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mouseEntered(final MouseEvent arg0) {
		// TODO Auto-generated method stub
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mouseExited(final MouseEvent arg0) {
		// TODO Auto-generated method stub
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mousePressed(final MouseEvent arg0) {
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mouseReleased(final MouseEvent arg0) {		
		//		if (arg0.getSource() instanceof JSlider) {
		//			if (arg0.getSource().equals(nofEventsSliderSupp.getSlider())) {
		//				//constraintsToRemoveSupp = new Vector();
		//				Vector adIds = new Vector();
		//			
		//				HashMap visible = getVisibleCd(((float)(((JSlider)arg0.getSource()).getValue()))/100f,templatesToRemove,eventsToRemove, etToRemove);
		//				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
		//					boolean found = false;
		//					for(Object key : visible.keySet()){		
		//						if((((HashMap)visible.get(key)).containsValue(cd))){
		//							found = true;
		//						}
		//					}
		//
		//					if(!found){
		//						decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
		//						cd.setVisible(false);
		//						//	constraintsToRemoveSupp.add(cd);
		//					}else{
		//						//	if(!constraintsToRemoveEve.contains(cd)&&!constraintsToRemoveTem.contains(cd)){
		//						cd.setVisible(true);
		//						decViz.getModel().getView().setConstraintDefinitionVisible(cd);
		//						float supp = 0f;
		//						float conf= 0f;
		//						 cpir= 0f;
		//						float ifact= 0f;
		//						if(cd.isVisible()){
		//							for(Object key : decViz.getVisiblesupportRule().keySet()){
		//								if(((Integer)key).equals(cd.getId())){
		//									if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
		//											&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
		//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
		//										conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
		//										cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
		//										ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
		//									}else{
		//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
		//										//	conf= Float.MAX_VALUE;
		//										//	cpir= Float.MAX_VALUE;
		//										//	ifact= Float.MAX_VALUE;
		//									}
		//								}
		//							}
		//							if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
		//									&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
		//
		//								if(sortingMetric.equals("cpir")){
		//									float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
		//									float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
		//									float normcpir = 1.f;
		//									if(maxcpir!=mincpir){
		//										normcpir=(cpir-mincpir)/(maxcpir-mincpir);
		//									}
		//									if(normcpir>1 || normcpir<0){
		//										normcpir=1;
		//									}
		//									Color cdCol;
		//									int green = 255-(int)(255*normcpir);
		//									cdCol = new Color(green,255,green);
		//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
		//									}else{
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
		//									}
		//								}else if(sortingMetric.equals("support")){
		//									float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
		//									float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
		//									float normsupp = 1.f;
		//									if(maxsupp!=minsupp){
		//										normsupp=(supp-minsupp)/(maxsupp-minsupp);
		//									}
		//									if(normsupp>1 || normsupp<0){
		//										normsupp=1;
		//									}
		//									Color cdCol;
		//									int green = 255-(int)(255*normsupp);
		//									cdCol = new Color(255,green,green);
		//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
		//									}else{
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
		//									}
		//								}else if(sortingMetric.equals("if")){
		//									float maxif = Collections.max(vizMod.getVisibleI().values());
		//									float minif = Collections.min(vizMod.getVisibleI().values());
		//									float normif = 1.f;
		//									if(maxif!=minif){
		//										normif=(ifact-minif)/(maxif-minif);
		//									}
		//									if(normif>1 || normif<0){
		//										normif=1;
		//									}
		//									Color cdCol;
		//									int green = 255-(int)(255*normif);
		//									cdCol = new Color(green,green,255);
		//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
		//									}else{
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
		//									}
		//								}else if(sortingMetric.equals("confidence")){
		//									float maxconf = Collections.max(vizMod.getVisibleconfidence().values());
		//									float minconf = Collections.min(vizMod.getVisibleconfidence().values());
		//									float normconf = (conf-minconf)/(maxconf-minconf);
		//									Color cdCol;
		//									if(normconf>1 || normconf<0){
		//										normconf=1;
		//									}
		//									int green = 255-(int)(255*normconf);
		//									cdCol = new Color(255,green,255);
		//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
		//									}else{
		//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
		//									}
		//								}
		//							}else{
		//								float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
		//								float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
		//								float normsupp = 1.f;
		//								if(maxsupp!=minsupp){
		//									normsupp=(supp-minsupp)/(maxsupp-minsupp);
		//								}
		//								Color cdCol;
		//								int green = 255-(int)(255*normsupp);
		//								cdCol = new Color(255,green,green);
		//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
		//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
		//								}else{
		//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
		//								}
		//							}
		//						}
		//						for(Parameter p: cd.getParameters()){
		//							adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
		//						}
		//					}
		//					//}
		//				}
		//				//activitiesToRemoveSupp = new Vector();
		//				for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
		//					if(!adIds.contains(ad.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
		//						//		activitiesToRemoveSupp.add(ad);
		//
		//						decViz.getModel().getView().setActivityDefinitionInvisible(ad);
		//						ad.setVisible(false);
		//					}else{
		//						//		if(!activitiesToRemoveEve.contains(ad)&&!activitiesToRemoveTem.contains(ad)){
		//						decViz.getModel().getView().setActivityDefinitionVisible(ad);
		//						ad.setVisible(true);
		//						//	}
		//					}
		//				}
		//
		//			}else{
		for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){

			//currentcp = ((float)(((JSlider)arg0.getSource()).getValue()))/100f;
			float supp = 0f;
			float conf= 0f;
			float cpir= 0f;
			float ifact= 0f;
			//	if(cd.isVisible()){
			for(Object key : decViz.getVisiblesupportRule().keySet()){
				if(((Integer)key).equals(cd.getId())){
					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
						supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
						conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
						cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
						ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
					}else{
						supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
						//	conf= Float.MAX_VALUE;
						//	cpir= Float.MAX_VALUE;
						//	ifact= Float.MAX_VALUE;
					}
				}
			}
			if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
					&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		

				if(sortingMetric.equals("cpir")){
					float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
					float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
					float normcpir = 1.f;
					if(maxcpir!=mincpir){
						normcpir=(cpir-mincpir)/(maxcpir-mincpir);
					}
					Color cdCol;
					if(normcpir>1 || normcpir<0){
						normcpir=1;
					}
					int green = 255-(int)(255*normcpir);
					cdCol = new Color(green,255,green);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
				}else if(sortingMetric.equals("support")){
					float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
					float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
					float normsupp = 1.f;
					if(maxsupp!=minsupp){
						normsupp=(supp-minsupp)/(maxsupp-minsupp);
					}
					if(normsupp>1 || normsupp<0){
						normsupp=1;
					}
					Color cdCol;
					int green = 255-(int)(255*normsupp);
					cdCol = new Color(255,green,green);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
				}else if(sortingMetric.equals("if")){
					float maxif = Collections.max(vizMod.getVisibleI().values());
					float minif = Collections.min(vizMod.getVisibleI().values());
					float normif = 1.f;
					if(maxif!=minif){
						normif=(ifact-minif)/(maxif-minif);
					}
					Color cdCol;
					if(normif>1 || normif<0){
						normif=1;
					}
					int green = 255-(int)(255*normif);
					cdCol = new Color(green,green,255);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
				}else if(sortingMetric.equals("confidence")){
					float maxconf = Collections.max(decViz.getVisibleconfidence().values());
					float minconf = Collections.min(decViz.getVisibleconfidence().values());
					float normconf = (conf-minconf)/(maxconf-minconf);
					Color cdCol;
					if(normconf>1 || normconf<0){
						normconf=1;
					}
					int green = 255-(int)(255*normconf);
					cdCol = new Color(255,green,255);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
				}
			}else{
				float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				float normsupp = 1.f;
				if(maxsupp!=minsupp){
					normsupp=(supp-minsupp)/(maxsupp-minsupp);
				}
				Color cdCol;
				int green = 255-(int)(255*normsupp);
				cdCol = new Color(255,green,green);
				if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
					decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				}else{
					decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				}
			}
			//	}
			//				}
			//			}
		}
		//		for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
		//			if(cd.getName().startsWith("existence") || cd.getName().startsWith("absence")|| cd.getName().startsWith("exactly")
		//					|| cd.getName().contains("choice") || cd.getName().contains("init")){
		//				if(cd.isVisible()){
		//					//	decViz.getModel().getView().setConstraintDefinitionColor(cd, Color.black, Color.black, Color.black);
		//				}
		//
		//			}
		//		}
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}

	}

	public JComponent showLogVis(final DeclareMinerOutput model, DeclareMinerInput input, XLog log) {
		initialize();
		this.extend = model.isExtend();
		this.decViz = model;
		this.input = input;
		this.log = log;
		completeGui();
		return this;
	}

	protected void completeGui() {
		DeclareMap decModel = decViz.getModel();
		float cpirmin = Float.MAX_VALUE;
		vizMod = new DeclareMinerOutput();
		vizMod.setVisibleCPIR(new HashMap());
		vizMod.setVisibleI(new HashMap());
		vizMod.setVisibleconfidence(new HashMap());
		if(decViz.getVisibleCPIR().size()>0){
			cpirmin = Collections.min(decViz.getVisibleCPIR().values());
			for(Integer o: decViz.getVisibleCPIR().keySet()){
				if((Float)decViz.getVisibleCPIR().get(o)>=Float.MAX_VALUE/10.f){
					vizMod.getVisibleCPIR().put(o, cpirmin);
				}else{
					vizMod.getVisibleCPIR().put(o, decViz.getVisibleCPIR().get(o));
				}
			}
		}


		float ifmin = Float.MAX_VALUE;
		if(decViz.getVisibleI().size()>0){
			ifmin = Collections.min(decViz.getVisibleI().values());
			for(Integer o: decViz.getVisibleI().keySet()){
				if((Float)decViz.getVisibleI().get(o)>=Float.MAX_VALUE/10.f){
					vizMod.getVisibleI().put(o, ifmin);
				}else{
					vizMod.getVisibleI().put(o, decViz.getVisibleI().get(o));
				}
			}
		}
		float confmin = Float.MAX_VALUE;
		if(decViz.getVisibleconfidence().size()>0){
			confmin = Collections.min(decViz.getVisibleconfidence().values());
			for(Integer o: decViz.getVisibleconfidence().keySet()){
				if((Float)decViz.getVisibleconfidence().get(o)>=Float.MAX_VALUE/10.f){
					vizMod.getVisibleconfidence().put(o, confmin);
				}else{
					vizMod.getVisibleconfidence().put(o, decViz.getVisibleconfidence().get(o));
				}
			}
		}
		AssignmentModelView view = decModel.getView();
		SlickerFactory sf = SlickerFactory.instance();
		mainPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		ProMSplitPane rsPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		rightPanel = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		leftPanel = sf.createRoundedPanel();
		//JPanel timePanel = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);


		SlickerTabbedPane tabbed = sf.createTabbedPane("", Color.YELLOW, Color.BLUE, Color.RED);
		tabbed.setBorder(BorderFactory.createTitledBorder("Filtering"));
		Vector labels = new Vector();
		JPanel rightTopPanel = sf.createRoundedPanel();
		rightTopPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {200, 150 , 75, 75}}));

		//rightTopPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL, TableLayoutConstants.FILL , TableLayoutConstants.FILL, TableLayoutConstants.FILL}}));

		int count = 0;
		for(Object n: decViz.getTemplate().values()){
			if(input.getSelectedDeclareTemplateSet().contains(n)){
				//String template =((ConstraintTemplate)n).getName();
				selectedTemplates.put(count, true);
				count++;
			}
			//values.add(v_checks[count]);

		}


		Vector cellListT = new Vector();
		for(Object n: decViz.getTemplate().values()){
			if(input.getSelectedDeclareTemplateSet().contains(n)){
				String template =(input.getDeclareTemplateConstraintTemplateMap().get(n)).getName();
				if(!temAdded.contains(template)){
					org.processmining.plugins.declareminer.visualizing.Cell cellL = new org.processmining.plugins.declareminer.visualizing.Cell();
					cellL.setLabel(template);
					//if(abstractionsToRemove.contains(abstraction+": "+field)){
					//	cellL.setSelected(false);
					//}else{
					cellL.setSelected(true);	
					//}
					cellListT.add(cellL);
					temAdded.add(template);
				}
			}
		}

		org.processmining.plugins.declareminer.visualizing.ListRenderer rendererTem = new org.processmining.plugins.declareminer.visualizing.ListRenderer();
		abstrList.setCellRenderer(rendererTem);
		abstrList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(cellListT));

		JPanel jp = new JPanel();
		abstrList.setBackground(jp.getBackground());

		abstrList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {

				int cellIndex = abstrList.locationToIndex(e.getPoint());
				//selectedIndex = cellIndex;
				if (!(Boolean) selectedTemplates.get(new Integer(cellIndex))) {
					selectedTemplates.put(new Integer(cellIndex).intValue(), true);
				} else {
					selectedTemplates.put(new Integer(cellIndex).intValue(), false);
				}
				Vector temp = new Vector();
				for (int i = 0; i < temAdded.size(); i++) {
					org.processmining.plugins.declareminer.visualizing.Cell cell = new org.processmining.plugins.declareminer.visualizing.Cell();
					if ((Boolean) selectedTemplates.get(new Integer(i))) {
						cell.setSelected(true);
						if(templatesToRemove.contains((String)temAdded.get(i))){
							templatesToRemove.remove((String)temAdded.get(i));
						}
					} else {
						cell.setSelected(false);
						if(!templatesToRemove.contains((String)temAdded.get(i))){
							templatesToRemove.add((String)temAdded.get(i));
						}
					}
					cell.setLabel((String)temAdded.get(i));
					temp.add(cell);
				}
				abstrList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(temp));



				//				String template = null;
				//
				//				Vector adIds = new Vector();
				//				HashMap visible = getVisibleCd(((float)((nofEventsSliderSupp.getSlider()).getValue()))/100f,templatesToRemove,eventsToRemove,etToRemove);
				//				//	constraintsToRemoveTem = new Vector();
				//				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//					boolean found = false;
				//					if(cd.getDisplay().equals("1..*")){
				//						template = "existence";
				//					}else if(cd.getDisplay().equals("2..*")){
				//						template = "existence2";
				//					}else if(cd.getDisplay().equals("3..*")){
				//						template = "existence3";
				//					}else if(cd.getDisplay().equals("1")){
				//						template = "exactly1";
				//					}else if(cd.getDisplay().equals("2")){
				//						template = "exactly2";
				//					}else if(cd.getDisplay().equals("3")){
				//						template = "exactly3";
				//					}else if(cd.getDisplay().equals("0")){
				//						template = "absence";
				//					}else if(cd.getDisplay().equals("0..1")){
				//						template = "absence2";
				//					}else if(cd.getDisplay().equals("0..2")){
				//						template = "absence3";
				//					}else if(cd.getDisplay().equals("choice")){
				//						template = "choice";
				//					}else if(cd.getDisplay().equals("init")){
				//						template = "init";
				//					}else if(cd.getDisplay().equals("exclusive choice")){
				//						template = "exclusive choice";
				//					}else{
				//						template = cd.getDisplay();
				//					}
				//					for(Object key : visible.keySet()){		
				//						if((((HashMap)visible.get(key)).containsValue(cd))&&!templatesToRemove.contains(template)){
				//							found = true;
				//						}
				//
				//					}
				//
				//					if(!found){
				//						decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//						cd.setVisible(false);
				//						//constraintsToRemoveTem.add(cd);
				//					}else{
				//						//	if(!constraintsToRemoveEve.contains(cd)&&!constraintsToRemoveSupp.contains(cd)){
				//						decViz.getModel().getView().setConstraintDefinitionVisible(cd);
				//						cd.setVisible(true);
				//						float supp = 0f;
				//						float conf= 0f;
				//						cpir= 0f;
				//						float ifact= 0f;
				//						if(cd.isVisible()){
				//							for(Object key : decViz.getVisiblesupportRule().keySet()){
				//								if(((Integer)key).equals(cd.getId())){
				//									if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//											&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
				//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//										conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
				//										cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
				//										ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
				//									}else{
				//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//										//	conf= Float.MAX_VALUE;
				//										//	cpir= Float.MAX_VALUE;
				//										//	ifact= Float.MAX_VALUE;
				//									}
				//								}
				//							}
				//							if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//									&& !cd.getName().contains("choice") && !cd.getName().contains("init")){	
				//								if(sortingMetric.equals("cpir")){
				//									float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
				//									float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
				//									float normcpir = 1.f;
				//									if(maxcpir!=mincpir){
				//										normcpir=(cpir-mincpir)/(maxcpir-mincpir);
				//									}
				//									Color cdCol;
				//									if(normcpir>1 || normcpir<0){
				//										normcpir=1;
				//									}
				//									int green = 255-(int)(255*normcpir);
				//									cdCol = new Color(green,255,green);
				//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//									}else{
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//									}
				//								}else if(sortingMetric.equals("support")){
				//									float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//									float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//									float normsupp = 1.f;
				//									if(maxsupp!=minsupp){
				//										normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//									}
				//									if(normsupp>1 || normsupp<0){
				//										normsupp=1;
				//									}
				//									Color cdCol;
				//									int green = 255-(int)(255*normsupp);
				//									cdCol = new Color(255,green,green);
				//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//									}else{
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//									}
				//								}else if(sortingMetric.equals("if")){
				//									float maxif = Collections.max(vizMod.getVisibleI().values());
				//									float minif = Collections.min(vizMod.getVisibleI().values());
				//									float normif = 1.f;
				//									if(maxif!=minif){
				//										normif=(ifact-minif)/(maxif-minif);
				//									}
				//									Color cdCol;
				//									if(normif>1 || normif<0){
				//										normif=1;
				//									}
				//									int green = 255-(int)(255*normif);
				//									cdCol = new Color(green,green,255);
				//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//									}else{
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//									}
				//								}else if(sortingMetric.equals("confidence")){
				//									float maxconf = Collections.max(decViz.getVisibleconfidence().values());
				//									float minconf = Collections.min(decViz.getVisibleconfidence().values());
				//									float normconf = (conf-minconf)/(maxconf-minconf);
				//									Color cdCol;
				//									if(normconf>1 || normconf<0){
				//										normconf=1;
				//									}
				//									int green = 255-(int)(255*normconf);
				//									cdCol = new Color(255,green,255);
				//									if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//									}else{
				//										decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//									}
				//								}
				//							}else{
				//								float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//								float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//								float normsupp = 1.f;
				//								if(maxsupp!=minsupp){
				//									normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//								}
				//								Color cdCol;
				//								int green = 255-(int)(255*normsupp);
				//								cdCol = new Color(255,green,green);
				//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//								}else{
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//								}
				//							}
				//						}
				//						for(Parameter p: cd.getParameters()){
				//							adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//						}
				//						//}
				//					}
				//				}
				//				//	activitiesToRemoveTem = new Vector();
				//				for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
				//					if(!adIds.contains(ad.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
				//
				//						decViz.getModel().getView().setActivityDefinitionInvisible(ad);
				//						ad.setVisible(false);
				//						//		activitiesToRemoveTem.add(ad);
				//					}else{
				//						//	if(!activitiesToRemoveEve.contains(ad)&&!activitiesToRemoveTem.contains(ad)){
				//						decViz.getModel().getView().setActivityDefinitionVisible(ad);
				//						ad.setVisible(true);
				//						//	}
				//					}
				//				}
				//
				//				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//					if(cd.getName().startsWith("existence") || cd.getName().startsWith("absence")|| cd.getName().startsWith("exactly")
				//							|| cd.getName().contains("choice") || cd.getName().contains("init")){
				//						if(cd.isVisible()){
				//							//	decViz.getModel().getView().setConstraintDefinitionColor(cd, Color.black, Color.black, Color.black);
				//						}
				//
				//					}
				//				}
				//
				//
				//				if (graph != null) {
				//					graph.refresh();
				//					graph.repaint();
				//				}
				//				if (mainPanel != null) {
				//					mainPanel.repaint();
				//				}
				//
				//				// We now have the index of the double-clicked cell..
				//				//}
			}
		});	


		JScrollPane scrollT = new JScrollPane(abstrList);
		scrollT.setOpaque(false);
		scrollT.getViewport().setOpaque(false);
		scrollT.setBorder(BorderFactory.createEmptyBorder());
		scrollT.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollT.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollT.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollT.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(scrollT.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollT.getHorizontalScrollBar().setOpaque(false);



		tabbed.addTab("templates", scrollT);



		int countE = 0;
		for(String ad: decViz.getAllActivities().values()){
		//for(int n=0; n<eventsList.; n++){	
		//String template =((ConstraintTemplate)n).getName();
			selectedEvents.put(countE, true);
			//values.add(v_checks[count]);
			countE++;
		}


		Vector cellListE = new Vector();
		for(String ad: decViz.getAllActivities().values()){
			String event =ad;
			if(!eveAdded.contains(event)){
				org.processmining.plugins.declareminer.visualizing.Cell cellL = new org.processmining.plugins.declareminer.visualizing.Cell();
				cellL.setLabel(event);
				//if(abstractionsToRemove.contains(abstraction+": "+field)){
				//	cellL.setSelected(false);
				//}else{
				cellL.setSelected(true);	
				//}
				cellListE.add(cellL);
				eveAdded.add(event);
			}
		}

		org.processmining.plugins.declareminer.visualizing.ListRenderer rendererEve = new org.processmining.plugins.declareminer.visualizing.ListRenderer();
		eventsList.setCellRenderer(rendererEve);
		eventsList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(cellListE));

		eventsList.setBackground(jp.getBackground());

		eventsList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {


				int cellIndex = eventsList.locationToIndex(e.getPoint());
				//selectedIndex = cellIndex;


				if (selectedEvents.get(new Integer(cellIndex))!= null){
					if(!(Boolean) selectedEvents.get(new Integer(cellIndex))) {
						selectedEvents.put(new Integer(cellIndex).intValue(), true);
					} else {
						selectedEvents.put(new Integer(cellIndex).intValue(), false);
					}
					Vector temp = new Vector();
					for (int i = 0; i < eveAdded.size(); i++) {
						org.processmining.plugins.declareminer.visualizing.Cell cell = new org.processmining.plugins.declareminer.visualizing.Cell();
						Integer inte = new Integer(i);
						//if(selectedEvents!=null && inte!=null && selectedEvents.get(inte)!=null){
							if (selectedEvents.get(inte)==null ||(Boolean) selectedEvents.get(inte)) {
								cell.setSelected(true);
								if(eventsToRemove.contains((String)eveAdded.get(i))){
									eventsToRemove.remove((String)eveAdded.get(i));
								}
							} else if (!(Boolean) selectedEvents.get(inte)) {
								cell.setSelected(false);
								if(!eventsToRemove.contains((String)eveAdded.get(i))){
									eventsToRemove.add((String)eveAdded.get(i));
								}
							}
						//}
					
					cell.setLabel((String)eveAdded.get(i));
					temp.add(cell);
					}

				
				eventsList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(temp));
				//				Vector adInvis = new Vector();
				//				//activitiesToRemoveEve = new Vector();
				//				//constraintsToRemoveEve = new Vector();
				//				for(ActivityDefinition actd: decViz.getModel().getModel().getActivityDefinitions()){
				//
				//					if(eventsToRemove.contains(actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
				//
				//
				//						//	activitiesToRemoveEve.add(actd);
				//						adInvis.add(actd);
				//						decViz.getModel().getView().setActivityDefinitionInvisible(actd);
				//						actd.setVisible(false);
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							for(Parameter p: cd.getParameters()){
				//								if(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").equals(actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
				//									decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//									cd.setVisible(false);
				//									//			constraintsToRemoveEve.add(cd);
				//								}
				//							}
				//						}
				//					}else{
				//						Vector adIds = new Vector();
				//						HashMap visible = getVisibleCd(((float)((nofEventsSliderSupp.getSlider()).getValue()))/100f,templatesToRemove, eventsToRemove, etToRemove);
				//						String template = null;
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							boolean found = false;
				//							if(cd.getDisplay().equals("1..*")){
				//								template = "existence";
				//							}else if(cd.getDisplay().equals("2..*")){
				//								template = "existence2";
				//							}else if(cd.getDisplay().equals("3..*")){
				//								template = "existence3";
				//							}else if(cd.getDisplay().equals("1")){
				//								template = "exactly1";
				//							}else if(cd.getDisplay().equals("2")){
				//								template = "exactly2";
				//							}else if(cd.getDisplay().equals("3")){
				//								template = "exactly3";
				//							}else if(cd.getDisplay().equals("0")){
				//								template = "absence";
				//							}else if(cd.getDisplay().equals("0..1")){
				//								template = "absence2";
				//							}else if(cd.getDisplay().equals("0..2")){
				//								template = "absence3";
				//							}else if(cd.getDisplay().equals("choice")){
				//								template = "choice";
				//							}else if(cd.getDisplay().equals("init")){
				//								template = "init";
				//							}else if(cd.getDisplay().equals("exclusive choice")){
				//								template = "exclusive choice";
				//							}else{
				//
				//								template = cd.getDisplay();
				//							}
				//							for(Object key : visible.keySet()){		
				//								if((((HashMap)visible.get(key)).containsValue(cd))&&!templatesToRemove.contains(template)){
				//									found = true;
				//								}
				//							}
				//							if(!found){
				//								decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//								cd.setVisible(false);
				//								//	constraintsToRemoveEve.add(cd);
				//							}else{
				//								//if(!constraintsToRemoveTem.contains(cd)&&!constraintsToRemoveSupp.contains(cd)){
				//								decViz.getModel().getView().setConstraintDefinitionVisible(cd);
				//								cd.setVisible(true);
				//								float supp = 0f;
				//								float conf= 0f;
				//								 cpir= 0f;
				//								float ifact= 0f;
				//								for(Object key : decViz.getVisiblesupportRule().keySet()){
				//									if(((Integer)key).equals(cd.getId())){
				//										if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//												&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
				//											supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//											conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
				//											cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
				//											ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
				//										}else{
				//											supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//											//conf= Float.MAX_VALUE;
				//											//cpir= Float.MAX_VALUE;
				//											//ifact= Float.MAX_VALUE;
				//										}
				//									}
				//								}
				//								if(cd.isVisible()){
				//									if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//											&& !cd.getName().contains("choice") && !cd.getName().contains("init")){	
				//										if(sortingMetric.equals("cpir")){
				//											float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
				//											float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
				//											float normcpir = 1.f;
				//											if(maxcpir!=mincpir){
				//												normcpir=(cpir-mincpir)/(maxcpir-mincpir);
				//											}
				//											Color cdCol;
				//											if(normcpir>1 || normcpir<0){
				//												normcpir=1;
				//											}
				//											int green = 255-(int)(255*normcpir);
				//											cdCol = new Color(green,255,green);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("support")){
				//											float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//											float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//											float normsupp = 1.f;
				//											if(maxsupp!=minsupp){
				//												normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//											}
				//											Color cdCol;
				//											if(normsupp>1 || normsupp<0){
				//												normsupp=1;
				//											}
				//											int green = 255-(int)(255*normsupp);
				//											cdCol = new Color(255,green,green);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("if")){
				//											float maxif = Collections.max(vizMod.getVisibleI().values());
				//											float minif = Collections.min(vizMod.getVisibleI().values());
				//											float normif = 1.f;
				//											if(maxif!=minif){
				//												normif=(ifact-minif)/(maxif-minif);
				//											}
				//											if(normif>1 || normif<0){
				//												normif=1;
				//											}
				//											Color cdCol;
				//											int green = 255-(int)(255*normif);
				//											cdCol = new Color(green,green,255);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("confidence")){
				//											float maxconf = Collections.max(decViz.getVisibleconfidence().values());
				//											float minconf = Collections.min(decViz.getVisibleconfidence().values());
				//											float normconf = (conf-minconf)/(maxconf-minconf);
				//											Color cdCol;
				//											if(normconf>1 || normconf<0){
				//												normconf=1;
				//											}
				//											int green = 255-(int)(255*normconf);
				//											cdCol = new Color(255,green,255);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}
				//									}else{
				//										float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//										float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//										float normsupp = 1.f;
				//										if(maxsupp!=minsupp){
				//											normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//										}
				//										Color cdCol;
				//										int green = 255-(int)(255*normsupp);
				//										cdCol = new Color(255,green,green);
				//										if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//											decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//										}else{
				//											decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//										}
				//									}
				//									for(Parameter p: cd.getParameters()){
				//										adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//									}
				//									//}
				//								}
				//							}
				//						}
				//
				//						for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
				//							if(!adIds.contains(ad.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
				//								decViz.getModel().getView().setActivityDefinitionInvisible(ad);
				//								ad.setVisible(false);
				//								//	activitiesToRemoveEve.add(ad);
				//							}else{
				//								if(!adInvis.contains(ad)){
				//									decViz.getModel().getView().setActivityDefinitionVisible(ad);
				//								}
				//							}
				//						}
				//
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							if(cd.getName().startsWith("existence") || cd.getName().startsWith("absence")|| cd.getName().startsWith("exactly")
				//									|| cd.getName().contains("choice") || cd.getName().contains("init")){
				//								if(cd.isVisible()){
				//									//	decViz.getModel().getView().setConstraintDefinitionColor(cd, Color.black, Color.black, Color.black);
				//								}
				//							}
				//						}
				//					}
				//
				//				}
				//
				//
				//				for(Object obj: decViz.getMappingAdCd().keySet()){
				//					Vector v = (Vector)decViz.getMappingAdCd().get(obj);
				//					boolean todelete = true;
				//					for(Object objCd: v){
				//						ConstraintDefinition cd = (ConstraintDefinition)objCd;
				//						if(decViz.getModel().getModel().constraintWithId(cd.getId())!=null){
				//							if(decViz.getModel().getModel().constraintWithId(cd.getId()).isVisible()){
				//								todelete = false;
				//							}
				//						}
				//					}
				//					if(todelete){
				//						adInvis.add((ActivityDefinition)obj);
				//						decViz.getModel().getView().setActivityDefinitionInvisible((ActivityDefinition)obj);
				//						((ActivityDefinition)obj).setVisible(false);
				//						//eventsToRemove.add(((ActivityDefinition)obj).getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//					}
				//				}
				//
				//
				//				if (graph != null) {
				//					graph.refresh();
				//					graph.repaint();
				//				}
				//				if (mainPanel != null) {
				//					mainPanel.repaint();
				//				}
				//
				//
				//				// We now have the index of the double-clicked cell..
				//}
			}
			}
		});	


		JScrollPane scrollE = new JScrollPane(eventsList);
		scrollE.setOpaque(false);
		scrollE.getViewport().setOpaque(false);
		scrollE.setBorder(BorderFactory.createEmptyBorder());
		scrollE.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollE.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollE.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollE.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(scrollE.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollE.getHorizontalScrollBar().setOpaque(false);

		tabbed.addTab("events", scrollE);


		JPanel rightTabbedPanel = sf.createRoundedPanel();

		rightTabbedPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL, TableLayoutConstants.FILL}}));

		rightTabbedPanel.add(tabbed,"0,0");



























		JList labelsList = new JList();
		ListRendererResult renderer = new ListRendererResult();
		labelsList.setCellRenderer(renderer);

		Vector cellList = new Vector();
		for (int i = 0; i < labels.size(); i++) {
			if(!((JLabel)labels.get(i)).getText().equals("Support Antecedent")&&!((JLabel)labels.get(i)).getText().equals("Support Consequent")){
				Cell cellL = new Cell();
				if (i == 0) {
					cellL.setCurrentSelected(true);
				}
				cellL.setLabel(((JLabel)labels.get(i)).getText());
				cellL.setSelected(false);
				cellList.add(cellL);
			}
		}
		labelsList.setModel(new CellModel(cellList));
		labelsList.setSelectionBackground(new Color(0, 100, 0));
		labelsList.setSelectedIndex(0); //It's easier to see the focus change
		//if an item is selected.
		labelsList.setBackground(new Color(160, 160, 160));
		labelsList.setSelectionForeground(Color.white);
		labelsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//final Vector tempTems = templs;

		JScrollPane scroll = new JScrollPane(labelsList);
		scroll.setOpaque(false);
		scroll.getViewport().setOpaque(false);
		scroll.setBorder(BorderFactory.createEmptyBorder());
		scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scroll.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scroll.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(scroll.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scroll.getHorizontalScrollBar().setOpaque(false);
		//rightPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL } }));
		rightTopPanel.add(scroll, "0,0");
		ButtonGroup bgroup = new ButtonGroup();
		final JRadioButton supportRB = sf.createRadioButton("Support");
		supportRB.setSelected(true);
		final JRadioButton cpirRB = sf.createRadioButton("CPIR");
		final JRadioButton confidenceRB = sf.createRadioButton("Confidence");
		final JRadioButton ifRB = sf.createRadioButton("Interest Factor");
		supportRB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				sortingMetric = "support";
				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
					//			if(cd.isVisible()){
					float supp = 0f;
					for(Object key : decViz.getVisiblesupportRule().keySet()){
						if(((Integer)key).equals(cd.getId())){
							//		if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							//			&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
							supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
							//	}
						}
					}
					float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
					float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
					float normsupp = 1.f;
					if(maxsupp!=minsupp){
						normsupp=(supp-minsupp)/(maxsupp-minsupp);
					}
					Color cdCol;
					int green = 255-(int)(255*normsupp);
					cdCol = new Color(255,green,green);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
					//	}
					if (graph != null) {
						graph.refresh();
						graph.repaint();
					}
					if (mainPanel != null) {
						mainPanel.repaint();
					}
				}
			}
		});
		cpirRB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				sortingMetric = "cpir";
				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){

					float cpir = 0f;
					for(Object key : decViz.getVisiblesupportRule().keySet()){
						if(((Integer)key).equals(cd.getId())){
							if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
									&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
								cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();

							}else{
								cpir =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
							}
						}
					}
					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		

						float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
						float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
						float normcpir = 1.f;
						if(maxcpir!=mincpir){
							normcpir= (cpir-mincpir)/(maxcpir-mincpir);
						}
						Color cdCol;
						if(normcpir>1 || normcpir<0){
							normcpir=1;
						}
						int green = 255-(int)(255*normcpir);
						cdCol = new Color(green,255,green);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}
					}else{
						float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
						float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
						float normsupp = 1.f;
						if(maxsupp!=minsupp){
							normsupp=(cpir-minsupp)/(maxsupp-minsupp);
						}
						if(normsupp>1 || normsupp<0){
							normsupp=1;
						}

						Color cdCol;
						int green = 255-(int)(255*normsupp);
						cdCol = new Color(255,green,green);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}					
					}

					if (graph != null) {
						graph.refresh();
						graph.repaint();
					}
					if (mainPanel != null) {
						mainPanel.repaint();
					}
				}
			}
		});
		confidenceRB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				sortingMetric = "confidence";	

				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
					float conf = 0f;
					for(Object key : decViz.getVisiblesupportRule().keySet()){
						if(((Integer)key).equals(cd.getId())){
							if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
									&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
								conf =((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
							}else{
								conf = ((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
							}
						}
					}
					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
						float maxconf = Collections.max(vizMod.getVisibleconfidence().values());
						float minconf = Collections.min(vizMod.getVisibleconfidence().values());
						float normconf = 1.f;
						if(maxconf!=minconf){
							normconf=(conf-minconf)/(maxconf-minconf);
						}

						Color cdCol;
						int green = 255-(int)(255*normconf);
						cdCol = new Color(255,green,255);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}
					}else{
						float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
						float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
						float normsupp = 1.f;
						if(maxsupp!=minsupp){
							normsupp=(conf-minsupp)/(maxsupp-minsupp);
						}
						Color cdCol;
						int green = 255-(int)(255*normsupp);
						cdCol = new Color(255,green,green);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}
					}
					if (graph != null) {
						graph.refresh();
						graph.repaint();
					}
					if (mainPanel != null) {
						mainPanel.repaint();
					}
				}
			}
		});
		ifRB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				sortingMetric = "if";
				float ifact = 0.f;
				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
					for(Object key : decViz.getVisiblesupportRule().keySet()){
						if(((Integer)key).equals(cd.getId())){
							if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
									&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
								ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
							}else{
								ifact = ((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
							}
						}
					}
					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		

						float maxif = Collections.max(vizMod.getVisibleI().values());
						float minif = Collections.min(vizMod.getVisibleI().values());
						float normif = 1.f;
						if(maxif!=minif){
							normif= (ifact-minif)/(maxif-minif);
						}
						Color cdCol;
						int green = 255-(int)(255*normif);
						cdCol = new Color(green,green,255);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}
					}else{
						float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
						float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
						float normsupp = 1.f;
						if(maxsupp!=minsupp){
							normsupp=(ifact-minsupp)/(maxsupp-minsupp);
						}
						Color cdCol;
						int green = 255-(int)(255*normsupp);
						cdCol = new Color(255,green,green);
						if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
							decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
						}else{
							decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
						}
					}
					if (graph != null) {
						graph.refresh();
						graph.repaint();
					}
					if (mainPanel != null) {
						mainPanel.repaint();
					}
				}
			}
		});
		bgroup.add(supportRB);
		bgroup.add(cpirRB);
		bgroup.add(confidenceRB);
		bgroup.add(ifRB);
		JPanel radioPanel = sf.createRoundedPanel();
		radioPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL,TableLayoutConstants.FILL,TableLayoutConstants.FILL,TableLayoutConstants.FILL } }));
		radioPanel.setBorder(BorderFactory.createTitledBorder("Sorting"));
		radioPanel.add(supportRB, "0,0");
		radioPanel.add(cpirRB, "0,1");
		radioPanel.add(confidenceRB, "0,2");
		radioPanel.add(ifRB, "0,3");
		Vector colNams = new Vector();
		colNams.add("Activations");
		colNams.add("Fulfilments");
		colNams.add("Violations");
		colNams.add("Conflicts");
		Vector data = new Vector();
		data.add(new Vector());
		rightTopPanel.add(radioPanel, "0,1");
		TableModel dtm = new DefaultTableModel(data,colNams);
		ProMTable pt = new ProMTable(dtm);
		rightTopPanel.add(pt,"0,2");
		colNams = new Vector();
		colNams.add("Max delay");
		colNams.add("Min delay");
		colNams.add("Avg delay");
		colNams.add("StdDev delay");
		data = new Vector();
		Vector row = new Vector();
		data.add(row);
		dtm = new DefaultTableModel(data,colNams);
		pt = new ProMTable(dtm);
		rightTopPanel.add(pt,"0,3");
		//rightPanel.add(sf.createRoundedPanel(), "0,4");


		graph = null;
		if ((decModel != null) && (decModel.getView() != null)) {
			graph = decModel.getView().getGraph();

			//			JGraphLayoutAlgorithm.applyLayout(graph,new TreeLayoutAlgorithm(),graph.getComponents());
			//final JScrollPane scroll = new JScrollPane(graph);
			graph.addMouseListener(this);
			graph.setBackground(Color.DARK_GRAY);

			////////////////////////////REMOVE///////////////////////
			//graph.setBackground(new Color(173,173,173));
			//////////////////////////REMOVE///////////////////////

			//scroll.addMouseListener(this);
			graph.refresh();
			graph.repaint();
			//	scroll.repaint();
			JScrollPane scrollpane = new JScrollPane(graph);
			scrollpane.setBackground(Color.DARK_GRAY);

			////////////////////////////REMOVE///////////////////////
			//			scrollpane.setBackground(new Color(173,173,173));
			//////////////////////////REMOVE///////////////////////

			scrollpane.setOpaque(false);
			scrollpane.getViewport().setOpaque(false);
			scrollpane.setBorder(BorderFactory.createEmptyBorder());
			scrollpane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			scrollpane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
			SlickerDecorator.instance().decorate(scrollpane.getVerticalScrollBar(), new Color(0, 0, 0, 0),
					new Color(140, 140, 140), new Color(80, 80, 80));
			scrollpane.getVerticalScrollBar().setOpaque(false);

			SlickerDecorator.instance().decorate(scrollpane.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
					new Color(140, 140, 140), new Color(80, 80, 80));
			scrollpane.getHorizontalScrollBar().setOpaque(false);
			leftPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL , 60}}));
			leftPanel.add(scrollpane, "0,0");





			float minSupportOnTheSlider = decViz.getMinSupport()*100f;
			float maxSupportOnTheSlider = minSupportOnTheSlider;
			if(decViz.getSupportRule().values().iterator().hasNext()){
				maxSupportOnTheSlider = Collections.max(decViz.getSupportRule().values())*100f;
			}
			//	minSupportOnTheSlider = minSupportOnTheSlider-1;
			//	maxSupportOnTheSlider = maxSupportOnTheSlider +1;

			float avgSupportOnTheSlider = ((minSupportOnTheSlider + minSupportOnTheSlider)/2.f);



			float maxConfidenceOnTheSlider = 0;
			for(Float max : decViz.getConfidence().values()){
				if((max > maxConfidenceOnTheSlider) && (max < 1000)){
					maxConfidenceOnTheSlider = max;
				}
			}

			float minConfidenceOnTheSlider = maxConfidenceOnTheSlider;
			if(decViz.getConfidence().values().iterator().hasNext()){
				minConfidenceOnTheSlider = Collections.min(decViz.getConfidence().values())*100f;
			}
			if(minConfidenceOnTheSlider >= (1000)){
				minConfidenceOnTheSlider = 0;
			}
			maxConfidenceOnTheSlider = maxConfidenceOnTheSlider*100f;
			//	minConfidenceOnTheSlider = minConfidenceOnTheSlider-1;
			//	maxConfidenceOnTheSlider = maxConfidenceOnTheSlider +1;

			float avgConfidenceOnTheSlider = ((minConfidenceOnTheSlider + minConfidenceOnTheSlider)/2.f);

			float maxCPIROnTheSlider = 0;
			for(Float max : decViz.getCPIR().values()){
				if((max > maxCPIROnTheSlider) && (max < 1000)){
					maxCPIROnTheSlider = max;
				}
			}

			float minCPIROnTheSlider =maxCPIROnTheSlider;
			if(decViz.getCPIR().values().iterator().hasNext()){
				minCPIROnTheSlider = Collections.min(decViz.getCPIR().values())*100f;
			}
			if(minCPIROnTheSlider >= (1000)){
				minCPIROnTheSlider = 0;
			}
			maxCPIROnTheSlider = maxCPIROnTheSlider*100f;
			minCPIROnTheSlider = minCPIROnTheSlider-1;
			maxCPIROnTheSlider = maxCPIROnTheSlider +1;

			float avgCPIROnTheSlider = ((minCPIROnTheSlider + minCPIROnTheSlider)/2.f);

			float maxInterestFactorOnTheSlider = 0;
			for(Float max : decViz.getI().values()){
				if((max > maxInterestFactorOnTheSlider) && (max < 1000)){
					maxInterestFactorOnTheSlider = max;
				}
			}

			float minInterestFactorOnTheSlider = maxInterestFactorOnTheSlider;
			if(decViz.getI().values().iterator().hasNext()){
				minInterestFactorOnTheSlider = Collections.min(decViz.getI().values())*100f;
			}
			if(minInterestFactorOnTheSlider >= (1000)){
				minInterestFactorOnTheSlider = 0;
			}
			maxInterestFactorOnTheSlider = maxInterestFactorOnTheSlider*100f;

			minInterestFactorOnTheSlider = minInterestFactorOnTheSlider-1;
			maxInterestFactorOnTheSlider = maxInterestFactorOnTheSlider +1;


			float avgInterestFactorOnTheSlider = ((minInterestFactorOnTheSlider + minInterestFactorOnTheSlider)/2.f);

			nofEventsSliderSupp = SlickerFactory.instance().createNiceIntegerSlider("Support", (int)minSupportOnTheSlider, (int)maxSupportOnTheSlider, (int)avgSupportOnTheSlider, Orientation.HORIZONTAL);
			nofEventsSliderCPIR = SlickerFactory.instance().createNiceIntegerSlider("CPIR", (int)minCPIROnTheSlider, (int)maxCPIROnTheSlider, (int)avgCPIROnTheSlider, Orientation.HORIZONTAL);
			nofEventsSliderConfidence = SlickerFactory.instance().createNiceIntegerSlider("Confidence", (int)minConfidenceOnTheSlider, (int)maxConfidenceOnTheSlider, (int)avgConfidenceOnTheSlider, Orientation.HORIZONTAL);
			nofEventsSliderInterestFactor = SlickerFactory.instance().createNiceIntegerSlider("IF", (int)minInterestFactorOnTheSlider, (int)maxInterestFactorOnTheSlider, (int)avgInterestFactorOnTheSlider, Orientation.HORIZONTAL);
			nofEventsSliderSupp.setPreferredSize(new Dimension(350, 20));
			nofEventsSliderSupp.setEnabled(true);
			//	nofEventsSliderSupp.getSlider().addMouseListener(this);
			//	leftPanel.add(nofEventsSliderSupp, "0,1");

			nofEventsSliderCPIR.setPreferredSize(new Dimension(350, 20));
			JPanel labelsPanel = sf.createRoundedPanel();
			JLabel activities = sf.createLabel("Number of activities in this map: " +decViz.getModel().getModel().activityDefinitionsCount());
			JLabel constraints = sf.createLabel("Number of constraints in this map: "+decViz.getModel().getModel().constraintDefinitionsCount());

			//activities.setBackground(Color.black);
			labelsPanel.setLayout(new TableLayout(new double[][] { { 300}, {30, 30}}));
			//labelsPanel.setBackground(Color.red);
			labelsPanel.add(activities,"0,0");
			labelsPanel.add(constraints,"0,1");
			leftPanel.add(labelsPanel, "0,1");
		}

		JButton regenerateModel = sf.createButton("Regenerate Model");
		regenerateModel.addMouseListener(new MouseListener() {

			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mousePressed(MouseEvent e) {


				float support = nofEventsSliderSupp.getSlider().getValue()/100.f;
				float confidence = nofEventsSliderConfidence.getSlider().getValue()/100.f;
				float cpir = nofEventsSliderCPIR.getSlider().getValue()/100.f;
				float interestFactor = nofEventsSliderInterestFactor.getSlider().getValue()/100.f;
				DeclareModelGenerator dmg = new DeclareModelGenerator();
				DeclareMinerOutput output = dmg.createModel(decViz.getActivityNameList(),decViz.isHier(), decViz.isTrans(), decViz, support, confidence, cpir, interestFactor, log, input, templatesToRemove,eventsToRemove, etToRemove, decViz.isExtend());
				decViz = output;

				float cpirmin = Float.MAX_VALUE;
				vizMod = new DeclareMinerOutput();
				vizMod.setVisibleCPIR(new HashMap());
				vizMod.setVisibleI(new HashMap());
				vizMod.setVisibleconfidence(new HashMap());
				if(decViz.getVisibleCPIR().size()>0){
					cpirmin = Collections.min(decViz.getVisibleCPIR().values());
					for(Integer o: decViz.getVisibleCPIR().keySet()){
						if((Float)decViz.getVisibleCPIR().get(o)>=Float.MAX_VALUE/10.f){
							vizMod.getVisibleCPIR().put(o, cpirmin);
						}else{
							vizMod.getVisibleCPIR().put(o, decViz.getVisibleCPIR().get(o));
						}
					}
				}


				float ifmin = Float.MAX_VALUE;
				if(decViz.getVisibleI().size()>0){
					ifmin = Collections.min(decViz.getVisibleI().values());
					for(Integer o: decViz.getVisibleI().keySet()){
						if((Float)decViz.getVisibleI().get(o)>=Float.MAX_VALUE/10.f){
							vizMod.getVisibleI().put(o, ifmin);
						}else{
							vizMod.getVisibleI().put(o, decViz.getVisibleI().get(o));
						}
					}
				}
				float confmin = Float.MAX_VALUE;
				if(decViz.getVisibleconfidence().size()>0){
					confmin = Collections.min(decViz.getVisibleconfidence().values());
					for(Integer o: decViz.getVisibleconfidence().keySet()){
						if((Float)decViz.getVisibleconfidence().get(o)>=Float.MAX_VALUE/10.f){
							vizMod.getVisibleconfidence().put(o, confmin);
						}else{
							vizMod.getVisibleconfidence().put(o, decViz.getVisibleconfidence().get(o));
						}
					}
				}
				supportRB.setSelected(true);

				cpirRB.setSelected(false);
				confidenceRB.setSelected(false);
				ifRB.setSelected(false);

				sortingMetric = "support";
				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
					//			if(cd.isVisible()){
					float supp = 0f;
					for(Object key : decViz.getVisiblesupportRule().keySet()){
						if(((Integer)key).equals(cd.getId())){
							//		if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							//			&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
							supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
							//	}
						}
					}
					float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
					float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
					float normsupp = 1.f;
					if(maxsupp!=minsupp){
						normsupp=(supp-minsupp)/(maxsupp-minsupp);
					}
					Color cdCol;
					int green = 255-(int)(255*normsupp);
					cdCol = new Color(255,green,green);
					if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
						decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
					}else{
						decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
					}
					//	}
					if (graph != null) {
						graph.refresh();
						graph.repaint();
					}
					if (mainPanel != null) {
						mainPanel.repaint();
					}
				}







				DeclareModelGenerator.layout(decViz.getModel().getView(), decViz.getModel().getModel());


				graph = null;
				SlickerFactory sf = SlickerFactory.instance();
				leftPanel = sf.createRoundedPanel();

				graph = decViz.getModel().getView().getGraph();
				//graph.addMouseListener(this);
				graph.setBackground(Color.DARK_GRAY);

				////////////////////////////REMOVE///////////////////////
				//				graph.setBackground(new Color(173,173,173));
				//////////////////////////REMOVE///////////////////////


				graph.refresh();
				graph.repaint();

				JScrollPane scrollpane = new JScrollPane(graph);
				scrollpane.setBackground(Color.DARK_GRAY);
				////////////////////////////REMOVE///////////////////////
				//			scrollpane.setBackground(new Color(173,173,173));
				//////////////////////////REMOVE///////////////////////
				scrollpane.setOpaque(false);
				scrollpane.getViewport().setOpaque(false);
				scrollpane.setBorder(BorderFactory.createEmptyBorder());
				scrollpane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
				scrollpane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
				SlickerDecorator.instance().decorate(scrollpane.getVerticalScrollBar(), new Color(0, 0, 0, 0),
						new Color(140, 140, 140), new Color(80, 80, 80));
				scrollpane.getVerticalScrollBar().setOpaque(false);

				SlickerDecorator.instance().decorate(scrollpane.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
						new Color(140, 140, 140), new Color(80, 80, 80));
				scrollpane.getHorizontalScrollBar().setOpaque(false);
				leftPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {TableLayoutConstants.FILL , 60}}));
				leftPanel.add(scrollpane, "0,0");

				JPanel labelsPanel = sf.createRoundedPanel();
				JLabel activities = sf.createLabel("Number of activities in this map: " +decViz.getModel().getModel().activityDefinitionsCount());
				JLabel constraints = sf.createLabel("Number of constraints in this map: "+decViz.getModel().getModel().constraintDefinitionsCount());
				//activities.setBackground(Color.black);
				labelsPanel.setLayout(new TableLayout(new double[][] { { 300}, {30, 30}}));
				//labelsPanel.setBackground(Color.red);
				labelsPanel.add(activities,"0,0");
				labelsPanel.add(constraints,"0,1");
				leftPanel.add(labelsPanel, "0,1");

				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){


					if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Init)){
						inits.add(cd);
					}
					if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Precedence)){
						precs.add(cd);
					}
					if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Responded_Existence)){
						responds.add(cd);
					}
					if(decViz.getAvgTD()!=null){
						decViz.getModel().getView().setConstraintDefinitionLabels(false,cd,mainPanel, rightPanel,decViz.getVisibleCPIR(), decViz.getVisiblesupportRule(),decViz.getSuppAntec(),decViz.getSupportConseq(),decViz.getVisibleconfidence(),decViz.getVisibleI(),decViz.getActivations(),decViz.getViolations(),decViz.getConflicts(),decViz.getFulfillments(),decViz.getMaxTD(),decViz.getMinTD(),decViz.getAvgTD(),decViz.getStdDevTD(), decViz.getConstraintIdTimeInstancesMap().get(cd.getId()));
					}else{
						decViz.getModel().getView().setConstraintDefinitionLabels(false,cd,mainPanel, rightPanel,decViz.getVisibleCPIR(), decViz.getVisiblesupportRule(),decViz.getSuppAntec(),decViz.getSupportConseq(),decViz.getVisibleconfidence(),decViz.getVisibleI(),null,null,null,null,null,null,null,null,null);
					}
				}
				//				Vector adIds = new Vector();
				//				HashMap visible = getVisibleCd(decViz.getMinSupport(),templatesToRemove, eventsToRemove, etToRemove);
				//				for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){
				//						nofEventsSliderCPIR.setEnabled(true);
				//					}
				//					boolean found = false;
				//					for(Object key : visible.keySet()){		
				//						if((((HashMap)visible.get(key)).containsValue(cd))){
				//							found = true;
				//						}
				//					}
				//					if(!found){
				//						decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//						cd.setVisible(false);
				//					}else{
				//						decViz.getModel().getView().setConstraintDefinitionVisible(cd);
				//						cd.setVisible(true);
				//						float supp = 0f;
				//						float conf= 0f;
				//						cpir= 0f;
				//						float ifact= 0f;
				//						if(cd.isVisible()){
				//							for(Object key : decViz.getVisiblesupportRule().keySet()){
				//								if(((Integer)key).equals(cd.getId())){
				//									if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//											&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
				//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//										conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
				//										cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
				//										ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
				//									}else{
				//										supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//									}
				//								}
				//							}
				//
				//							if(sortingMetric.equals("cpir")){
				//								float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
				//								float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
				//								float normcpir = 1.f;
				//								if(maxcpir!=mincpir){
				//									normcpir=(cpir-mincpir)/(maxcpir-mincpir);
				//								}
				//								Color cdCol;
				//								if(normcpir>1 || normcpir<0){
				//									normcpir = 1;
				//								}
				//								int green = 255-(int)(255*normcpir);
				//								cdCol = new Color(green,255,green);
				//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//								}else{
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//								}
				//							}else if(sortingMetric.equals("support")){
				//								float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//								float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//								float normsupp = 1.f;
				//								if(maxsupp!=minsupp){
				//									normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//								}
				//								Color cdCol;
				//								if(normsupp>1 || normsupp<0){
				//									normsupp = 1;
				//								}
				//								int green = 255-(int)(255*normsupp);
				//								cdCol = new Color(255,green,green);
				//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//								}else{
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//								}
				//							}else if(sortingMetric.equals("if")){
				//								float maxif = Collections.max(vizMod.getVisibleI().values());
				//								float minif = Collections.min(vizMod.getVisibleI().values());
				//								float normif = 1.f;
				//								if(maxif!=minif){
				//									normif=(ifact-minif)/(maxif-minif);
				//								}
				//								if(normif>1 || normif<0){
				//									normif= 1;
				//								}
				//								Color cdCol;
				//								int green = 255-(int)(255*normif);
				//								cdCol = new Color(green,green,255);
				//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//								}else{
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//								}
				//							}else if(sortingMetric.equals("confidence")){
				//								float maxconf = Collections.max(decViz.getVisibleconfidence().values());
				//								float minconf = Collections.min(decViz.getVisibleconfidence().values());
				//								float normconf = (conf-minconf)/(maxconf-minconf);
				//								Color cdCol;
				//								if(normconf>1 || normconf<0){
				//									normconf = 1;
				//								}
				//								int green = 255-(int)(255*normconf);
				//								cdCol = new Color(255,green,255);
				//								if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//								}else{
				//									decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//								}
				//							}
				//						}
				//						for(Parameter p: cd.getParameters()){
				//							adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//						}
				//					}
				//
				//				}
				for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
					if(!eventTypes.contains(ad.getName())){
						eventTypes.add(ad.getName());
					}
					decViz.getModel().getView().setActivityDefinitionLabels(ad, mainPanel, rightPanel, decViz.getActSupp(),decViz.getBlnc());
				}				

				final JGraphOrganicLayout oc = new JGraphOrganicLayout();

				oc.setDeterministic(true);
				oc.setOptimizeBorderLine(true);
				oc.setOptimizeEdgeCrossing(true);
				oc.setOptimizeEdgeDistance(true);
				oc.setOptimizeEdgeLength(true);
				oc.setOptimizeNodeDistribution(true);
				oc.setEdgeCrossingCostFactor(999999999);
				oc.setEdgeDistanceCostFactor(999999999);
				oc.setFineTuning(true);

				//	oc.setMinDistanceLimit(0.001);
				oc.setEdgeLengthCostFactor(9999);
				if(decViz.getModel().getModel().getConstraintDefinitions().size()<200){
					oc.setEdgeLengthCostFactor(99);
				}
				oc.setNodeDistributionCostFactor(999999999);
				oc.setBorderLineCostFactor(999);
				oc.setRadiusScaleFactor(0.9);
				final JGraphFacade jgf = new JGraphFacade(graph);
				oc.run(jgf);
				final Map nestedMap = jgf.createNestedMap(true, true); 
				graph.getGraphLayoutCache().edit(nestedMap); 


				if (leftPanel != null) {
					leftPanel.repaint();
				}


				leftPanel.revalidate();
				leftPanel.repaint();

				if (graph != null) {
					graph.refresh();
					graph.repaint();
				}



				mainPanel.setLeftComponent(leftPanel);
				mainPanel.repaint();
				revalidate();
				repaint();

			}

			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseEntered(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseClicked(MouseEvent e) {
				// TODO Auto-generated method stub

			}
		});
		JPanel slidersPanel = sf.createRoundedPanel();

		slidersPanel.setLayout(new TableLayout(new double[][] { { TableLayoutConstants.FILL}, {50,  50, 50, 50, 50}}));
		slidersPanel.add(nofEventsSliderSupp,"0,0");
		slidersPanel.add(nofEventsSliderConfidence, "0,1");
		slidersPanel.add(nofEventsSliderCPIR, "0,2");
		slidersPanel.add(nofEventsSliderInterestFactor, "0,3");
		slidersPanel.add(regenerateModel, "0,4");
		rightTabbedPanel.add(slidersPanel, "0,1");

		for(ConstraintDefinition cd: decModel.getModel().getConstraintDefinitions()){


			if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Init)){
				inits.add(cd);
			}
			if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Precedence)){
				precs.add(cd);
			}
			if(decViz.getTemplate().get(cd.getId()).equals(DeclareTemplate.Responded_Existence)){
				responds.add(cd);
			}
			if(decViz.getAvgTD()!=null){
				decModel.getView().setConstraintDefinitionLabels(false,cd,mainPanel, rightPanel,decViz.getVisibleCPIR(), decViz.getVisiblesupportRule(),decViz.getSuppAntec(),decViz.getSupportConseq(),decViz.getVisibleconfidence(),decViz.getVisibleI(),decViz.getActivations(),decViz.getViolations(),decViz.getConflicts(),decViz.getFulfillments(),decViz.getMaxTD(),decViz.getMinTD(),decViz.getAvgTD(),decViz.getStdDevTD(), decViz.getConstraintIdTimeInstancesMap().get(cd.getId()));
			}else{
				decModel.getView().setConstraintDefinitionLabels(false, cd,mainPanel, rightPanel,decViz.getVisibleCPIR(), decViz.getVisiblesupportRule(),decViz.getSuppAntec(),decViz.getSupportConseq(),decViz.getVisibleconfidence(),decViz.getVisibleI(),null,null,null,null,null,null,null,null,null);
			}
		}
		Vector adIds = new Vector();
		//		HashMap visible = getVisibleCd(decViz.getMinSupport(),templatesToRemove, eventsToRemove, etToRemove);
		for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
			//			if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
			//					&& !cd.getName().contains("choice") && !cd.getName().contains("init")){
			//				nofEventsSliderCPIR.setEnabled(true);
			//			}
			//			boolean found = false;
			//			for(Object key : visible.keySet()){		
			//				if((((HashMap)visible.get(key)).containsValue(cd))){
			//					found = true;
			//				}
			//			}
			//			if(!found){
			//				decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
			//				cd.setVisible(false);
			//			}else{
			//	decViz.getModel().getView().setConstraintDefinitionVisible(cd);
			//	cd.setVisible(true);
			float supp = 0f;
			float conf= 0f;
			float cpir= 0f;
			float ifact= 0f;
			//		if(cd.isVisible()){
			for(Object key : decViz.getVisiblesupportRule().keySet()){
				if(((Integer)key).equals(cd.getId())){
					if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
							&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
						supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
						conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
						cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
						ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
					}else{
						supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
						//	conf= Float.MAX_VALUE;
						//	cpir= Float.MAX_VALUE;
						//	ifact= Float.MAX_VALUE;
					}
				}
			}

			if(sortingMetric.equals("cpir")){
				float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
				float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
				float normcpir = 1.f;
				if(maxcpir!=mincpir){
					normcpir=(cpir-mincpir)/(maxcpir-mincpir);
				}
				Color cdCol;
				if(normcpir>1 || normcpir<0){
					normcpir = 1;
				}
				int green = 255-(int)(255*normcpir);
				cdCol = new Color(green,255,green);
				if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
					decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				}else{
					decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				}
			}else if(sortingMetric.equals("support")){
				float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				float normsupp = 1.f;
				if(maxsupp!=minsupp){
					normsupp=(supp-minsupp)/(maxsupp-minsupp);
				}
				if(normsupp>1 || normsupp<0){
					normsupp = 1;
				}
				Color cdCol;
				int green = 255-(int)(255*normsupp);
				cdCol = new Color(255,green,green);
				if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
					decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				}else{
					decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				}
			}else if(sortingMetric.equals("if")){
				float maxif = Collections.max(vizMod.getVisibleI().values());
				float minif = Collections.min(vizMod.getVisibleI().values());
				float normif = 1.f;
				if(maxif!=minif){
					normif=(ifact-minif)/(maxif-minif);
				}
				if(normif>1 || normif<0){
					normif = 1;
				}
				Color cdCol;
				int green = 255-(int)(255*normif);
				cdCol = new Color(green,green,255);
				if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
					decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				}else{
					decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				}
			}else if(sortingMetric.equals("confidence")){
				float maxconf = Collections.max(decViz.getVisibleconfidence().values());
				float minconf = Collections.min(decViz.getVisibleconfidence().values());
				float normconf = (conf-minconf)/(maxconf-minconf);
				Color cdCol;
				if(normconf>1 || normconf<0){
					normconf = 1;
				}
				int green = 255-(int)(255*normconf);
				cdCol = new Color(255,green,255);
				if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
					decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				}else{
					decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				}
			}
			//	}
			for(Parameter p: cd.getParameters()){
				if(cd.getBranches(p).iterator().hasNext()){
					adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				}else{
					adIds.add("EMPTY_PARAM");
				}
			}
		}

		//		}
		DeclareModelGenerator dmg = new DeclareModelGenerator();
		for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
			if(dmg.hasEventTypeInName(ad.getName())){
				String type = dmg.getEventType(ad.getName());
				if(!eventTypes.contains(type)){
					eventTypes.add(type);
				}	

			}

			decModel.getView().setActivityDefinitionLabels(ad, mainPanel, rightPanel, decViz.getActSupp(),decViz.getBlnc());
			//			if(!adIds.contains(ad.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
			//				decViz.getModel().getView().setActivityDefinitionInvisible(ad);
			//				ad.setVisible(false);
			//			}else{
			//				decViz.getModel().getView().setActivityDefinitionVisible(ad);
			//				ad.setVisible(true);
			//			}
		}
		//
		//		for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
		//			if(cd.getName().startsWith("existence") || cd.getName().startsWith("absence")|| cd.getName().startsWith("exactly")
		//					|| cd.getName().contains("choice") || cd.getName().contains("init")){
		//				if(cd.isVisible()){
		//					//decViz.getModel().getView().setConstraintDefinitionColor(cd, Color.black, Color.black, Color.black);
		//				}
		//
		//			}
		//		}
		//






		int countET = 0;
		for(Object n: eventTypes){
			//String template =((ConstraintTemplate)n).getName();
			selectedET.put(countET, true);
			//values.add(v_checks[count]);
			countET++;
		}


		Vector cellListET = new Vector();
		for(Object n: eventTypes){
			if(!etAdded.contains(n)){
				org.processmining.plugins.declareminer.visualizing.Cell cellL = new org.processmining.plugins.declareminer.visualizing.Cell();
				cellL.setLabel((String)n);
				//if(abstractionsToRemove.contains(abstraction+": "+field)){
				//	cellL.setSelected(false);
				//}else{
				cellL.setSelected(true);	
				//}
				cellListET.add(cellL);
				etAdded.add(n);
			}
		}

		org.processmining.plugins.declareminer.visualizing.ListRenderer rendererET = new org.processmining.plugins.declareminer.visualizing.ListRenderer();
		eventsTypList.setCellRenderer(rendererET);
		eventsTypList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(cellListET));

		eventsTypList.setBackground(jp.getBackground());

		eventsTypList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {

				//  if (e.getClickCount() == 2) {
				int cellIndex = eventsTypList.locationToIndex(e.getPoint());
				//selectedIndex = cellIndex;

				if (!(Boolean) selectedET.get(new Integer(cellIndex))) {
					selectedET.put(new Integer(cellIndex).intValue(), true);
				} else {
					selectedET.put(new Integer(cellIndex).intValue(), false);
				}
				Vector temp = new Vector();
				for (int i = 0; i < etAdded.size(); i++) {
					org.processmining.plugins.declareminer.visualizing.Cell cell = new org.processmining.plugins.declareminer.visualizing.Cell();
					if ((Boolean) selectedET.get(new Integer(i))) {
						cell.setSelected(true);
						if(etToRemove.contains((String)etAdded.get(i))){
							etToRemove.remove((String)etAdded.get(i));
						}
					} else {
						cell.setSelected(false);
						if(!etToRemove.contains((String)etAdded.get(i))){
							etToRemove.add((String)etAdded.get(i));
						}
					}
					cell.setLabel((String)etAdded.get(i));
					temp.add(cell);
				}

				eventsTypList.setModel(new org.processmining.plugins.declareminer.visualizing.CellModel(temp));
				//				Vector adInvis = new Vector();
				//				//activitiesToRemoveEve = new Vector();
				//				//constraintsToRemoveEve = new Vector();
				//				for(ActivityDefinition actd: decViz.getModel().getModel().getActivityDefinitions()){
				//
				//					if(etToRemove.contains(actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1])){
				//
				//						//	activitiesToRemoveEve.add(actd);
				//						adInvis.add(actd);
				//						decViz.getModel().getView().setActivityDefinitionInvisible(actd);
				//						actd.setVisible(false);
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							for(Parameter p: cd.getParameters()){
				//								if(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1].equals(actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[actd.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1])){
				//									decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//									cd.setVisible(false);
				//									//			constraintsToRemoveEve.add(cd);
				//								}
				//							}
				//						}
				//					}else{
				//						Vector adIds = new Vector();
				//						HashMap visible = getVisibleCd(((float)((nofEventsSliderSupp.getSlider()).getValue()))/100f,templatesToRemove, eventsToRemove, etToRemove);
				//						String template = null;
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							boolean found = false;
				//							if(cd.getDisplay().equals("1..*")){
				//								template = "existence";
				//							}else if(cd.getDisplay().equals("2..*")){
				//								template = "existence2";
				//							}else if(cd.getDisplay().equals("3..*")){
				//								template = "existence3";
				//							}else if(cd.getDisplay().equals("1")){
				//								template = "exactly1";
				//							}else if(cd.getDisplay().equals("2")){
				//								template = "exactly2";
				//							}else if(cd.getDisplay().equals("3")){
				//								template = "exactly3";
				//							}else if(cd.getDisplay().equals("0")){
				//								template = "absence";
				//							}else if(cd.getDisplay().equals("0..1")){
				//								template = "absence2";
				//							}else if(cd.getDisplay().equals("0..2")){
				//								template = "absence3";
				//							}else if(cd.getDisplay().equals("choice")){
				//								template = "choice";
				//							}else if(cd.getDisplay().equals("init")){
				//								template = "init";
				//							}else if(cd.getDisplay().equals("exclusive choice")){
				//								template = "exclusive choice";
				//							}else{
				//
				//								template = cd.getDisplay();
				//							}
				//							for(Object key : visible.keySet()){		
				//								if((((HashMap)visible.get(key)).containsValue(cd))&&!templatesToRemove.contains(template)){
				//									found = true;
				//								}
				//							}
				//							if(!found){
				//								decViz.getModel().getView().setConstraintDefinitionInvisible(cd);
				//								cd.setVisible(false);
				//								//	constraintsToRemoveEve.add(cd);
				//							}else{
				//								//if(!constraintsToRemoveTem.contains(cd)&&!constraintsToRemoveSupp.contains(cd)){
				//								decViz.getModel().getView().setConstraintDefinitionVisible(cd);
				//								cd.setVisible(true);
				//								float supp = 0f;
				//								float conf= 0f;
				//								 cpir= 0f;
				//								float ifact= 0f;
				//								for(Object key : decViz.getVisiblesupportRule().keySet()){
				//									if(((Integer)key).equals(cd.getId())){
				//										if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//												&& !cd.getName().contains("choice") && !cd.getName().contains("init")){		
				//											supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//											conf= ((Float)vizMod.getVisibleconfidence().get(key)).floatValue();
				//											cpir= ((Float)vizMod.getVisibleCPIR().get(key)).floatValue();
				//											ifact= ((Float)vizMod.getVisibleI().get(key)).floatValue();
				//										}else{
				//											supp =((Float)decViz.getVisiblesupportRule().get(key)).floatValue();
				//											//conf= Float.MAX_VALUE;
				//											//cpir= Float.MAX_VALUE;
				//											//ifact= Float.MAX_VALUE;
				//										}
				//									}
				//								}
				//								if(cd.isVisible()){
				//									if(!cd.getName().startsWith("existence") && !cd.getName().startsWith("absence")&& !cd.getName().startsWith("exactly")
				//											&& !cd.getName().contains("choice") && !cd.getName().contains("init")){	
				//										if(sortingMetric.equals("cpir")){
				//											float maxcpir = Collections.max(vizMod.getVisibleCPIR().values());
				//											float mincpir = Collections.min(vizMod.getVisibleCPIR().values());
				//											float normcpir = 1.f;
				//											if(maxcpir!=mincpir){
				//												normcpir=(cpir-mincpir)/(maxcpir-mincpir);
				//											}
				//											if(normcpir>1 || normcpir<0){
				//												normcpir = 1;
				//											}
				//											Color cdCol;
				//											int green = 255-(int)(255*normcpir);
				//											cdCol = new Color(green,255,green);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("support")){
				//											float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//											float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//											float normsupp = 1.f;
				//											if(maxsupp!=minsupp){
				//												normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//											}
				//											if(normsupp>1 || normsupp<0){
				//												normsupp = 1;
				//											}
				//											Color cdCol;
				//											int green = 255-(int)(255*normsupp);
				//											cdCol = new Color(255,green,green);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("if")){
				//											float maxif = Collections.max(vizMod.getVisibleI().values());
				//											float minif = Collections.min(vizMod.getVisibleI().values());
				//											float normif = 1.f;
				//											if(maxif!=minif){
				//												normif=(ifact-minif)/(maxif-minif);
				//											}
				//											if(normif>1 || normif<0){
				//												normif = 1;
				//											}
				//											Color cdCol;
				//											int green = 255-(int)(255*normif);
				//											cdCol = new Color(green,green,255);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}else if(sortingMetric.equals("confidence")){
				//											float maxconf = Collections.max(decViz.getVisibleconfidence().values());
				//											float minconf = Collections.min(decViz.getVisibleconfidence().values());
				//											float normconf = (conf-minconf)/(maxconf-minconf);
				//											Color cdCol;
				//											if(normconf>1 || normconf<0){
				//												normconf = 1;
				//											}
				//											int green = 255-(int)(255*normconf);
				//											cdCol = new Color(255,green,255);
				//											if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//											}else{
				//												decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//											}
				//										}
				//									}else{
				//										float maxsupp = Collections.max(decViz.getVisiblesupportRule().values());
				//										float minsupp = Collections.min(decViz.getVisiblesupportRule().values());
				//										float normsupp = 1.f;
				//										if(maxsupp!=minsupp){
				//											normsupp=(supp-minsupp)/(maxsupp-minsupp);
				//										}
				//										Color cdCol;
				//										int green = 255-(int)(255*normsupp);
				//										cdCol = new Color(255,green,green);
				//										if(cd.getName().contains("alternate")||cd.getName().contains("chain")){
				//											decViz.getModel().getView().setConstraintDefinitionColor(cd,Color.black,cdCol,cdCol);
				//										}else{
				//											decViz.getModel().getView().setConstraintDefinitionColor(cd,cdCol,cdCol,cdCol);
				//										}
				//									}
				//									for(Parameter p: cd.getParameters()){
				//										adIds.add(cd.getBranches(p).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//									}
				//									//}
				//								}
				//							}
				//						}
				//
				//						for(ActivityDefinition ad: decViz.getModel().getModel().getActivityDefinitions()){
				//							if(!adIds.contains(ad.getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"))){
				//								decViz.getModel().getView().setActivityDefinitionInvisible(ad);
				//								ad.setVisible(false);
				//								//	activitiesToRemoveEve.add(ad);
				//							}else{
				//								if(!adInvis.contains(ad)){
				//									decViz.getModel().getView().setActivityDefinitionVisible(ad);
				//									ad.setVisible(true);
				//								}
				//							}
				//						}
				//
				//						for(ConstraintDefinition cd: decViz.getModel().getModel().getConstraintDefinitions()){
				//							if(cd.getName().startsWith("existence") || cd.getName().startsWith("absence")|| cd.getName().startsWith("exactly")
				//									|| cd.getName().contains("choice") || cd.getName().contains("init")){
				//								if(cd.isVisible()){
				//									//	decViz.getModel().getView().setConstraintDefinitionColor(cd, Color.black, Color.black, Color.black);
				//								}
				//							}
				//						}
				//					}
				//
				//				}
				//
				//
				//				for(Object obj: decViz.getMappingAdCd().keySet()){
				//					Vector v = (Vector)decViz.getMappingAdCd().get(obj);
				//					boolean todelete = true;
				//					for(Object objCd: v){
				//						ConstraintDefinition cd = (ConstraintDefinition)objCd;
				//						if(decViz.getModel().getModel().constraintWithId(cd.getId())!=null){
				//							if(decViz.getModel().getModel().constraintWithId(cd.getId()).isVisible()){
				//								todelete = false;
				//							}
				//						}
				//					}
				//					if(todelete){
				//						adInvis.add((ActivityDefinition)obj);
				//						decViz.getModel().getView().setActivityDefinitionInvisible((ActivityDefinition)obj);
				//						((ActivityDefinition)obj).setVisible(false);
				//						//eventsToRemove.add(((ActivityDefinition)obj).getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>"));
				//					}
				//				}
				//
				//
				//				if (graph != null) {
				//					graph.refresh();
				//					graph.repaint();
				//				}
				//				if (mainPanel != null) {
				//					mainPanel.repaint();
				//				}
				//
				//
				//				// We now have the index of the double-clicked cell..
				//				//}
			}
		});	


		JScrollPane scrollET = new JScrollPane(eventsTypList);
		scrollE.setOpaque(false);
		scrollE.getViewport().setOpaque(false);
		scrollE.setBorder(BorderFactory.createEmptyBorder());
		scrollE.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollE.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollE.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollE.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(scrollE.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollE.getHorizontalScrollBar().setOpaque(false);

		tabbed.addTab("event types", scrollET);












		final JGraphOrganicLayout oc = new JGraphOrganicLayout();

		oc.setDeterministic(true);
		oc.setOptimizeBorderLine(true);
		oc.setOptimizeEdgeCrossing(true);
		oc.setOptimizeEdgeDistance(true);
		oc.setOptimizeEdgeLength(true);
		oc.setOptimizeNodeDistribution(true);
		oc.setEdgeCrossingCostFactor(999999999);
		oc.setEdgeDistanceCostFactor(999999999);
		oc.setFineTuning(true);

		//	oc.setMinDistanceLimit(0.001);
		oc.setEdgeLengthCostFactor(9999);
		if(decViz.getModel().getModel().getConstraintDefinitions().size()<200){
			oc.setEdgeLengthCostFactor(99);
		}
		oc.setNodeDistributionCostFactor(999999999);
		oc.setBorderLineCostFactor(999);
		oc.setRadiusScaleFactor(0.9);
		final JGraphFacade jgf = new JGraphFacade(graph);
		oc.run(jgf);
		final Map nestedMap = jgf.createNestedMap(true, true); 
		graph.getGraphLayoutCache().edit(nestedMap); 





		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}

		//mainPanel.add(leftPanel, "0,0");
		//mainPanel.add(rightPanel, "1,0");
		rightTopPanel.setSize(leftPanel.getSize());
		rightTopPanel.setPreferredSize(leftPanel.getSize());
		rightTopPanel.setMinimumSize(leftPanel.getSize());
		rightTopPanel.setMaximumSize(leftPanel.getSize());
		JScrollPane rightTopPanelScroll = new JScrollPane(rightTopPanel);  
		rightTopPanelScroll.setOpaque(false);
		rightTopPanelScroll.getViewport().setOpaque(false);
		rightTopPanelScroll.setBorder(BorderFactory.createEmptyBorder());
		rightTopPanelScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		rightTopPanelScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(rightTopPanelScroll.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		rightTopPanelScroll.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(rightTopPanelScroll.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		rightTopPanelScroll.getHorizontalScrollBar().setOpaque(false);


		rightTopPanelScroll.setSize(new Dimension(300, 300));
		rightTopPanelScroll.setPreferredSize(new Dimension(300, 300));
		rightTopPanelScroll.setMinimumSize(new Dimension(300, 300));
		rightTopPanelScroll.setMaximumSize(new Dimension(300, 300));
		System.out.println("I am in the visualization");
		for(Integer index: decViz.getTemplate().keySet()){
			if(decViz.getTemplate().get(index).equals(DeclareTemplate.Response)){
				System.out.println("interest factor"+  decViz.getConstraintParametersMap().get(index)+": "+decViz.getI().get(index));		
			}
		}

		rightPanel.setTopComponent(rightTopPanelScroll);
		rightPanel.setBottomComponent(sf.createRoundedPanel());
		rsPanel.setLeftComponent(rightPanel);
		rsPanel.setRightComponent(rightTabbedPanel);
		mainPanel.setLeftComponent(leftPanel);
		mainPanel.setResizeWeight(1.0);
		mainPanel.setOneTouchExpandable(true);
		mainPanel.setRightComponent(rsPanel);
		add(mainPanel);
		revalidate();
		repaint();
	}

	/**
	 * loads the log and initializes this component
	 */
	protected void initialize() {
		String promDir = System.getProperty("user.dir");
		if (promDir.endsWith(".")) {
			promDir = promDir.substring(0, promDir.length() - 1);
		}
		if (promDir.endsWith(System.getProperty("file.separator")) == false) {
			promDir += System.getProperty("file.separator");
		}
		setBackground(new Color(40, 40, 40));

		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());

	}

	//	public HashMap getVisibleCd(float minSupp, Vector toSkip /*templates*/, Vector toHide, Vector toETHide){
	//		HashMap visibleCds = new HashMap();
	//		if(extend){
	//			for(ConstraintDefinition cd: this.decViz.getModel().getModel().getConstraintDefinitions()){
	//				String paramA = null;
	//				String paramB =null;
	//				paramA = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>");
	//				if(cd.getParameterWithId(2)!= null){
	//					paramB = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>");
	//				}
	//				String comp1 = ""; 
	//				if(cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>").length>1){
	//					comp1 = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>")[1].split("</center></html>")[0];
	//				}else{
	//					comp1 = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1];			
	//				}
	//				if(!toSkip.contains(cd.getName())&&!toHide.contains(paramA)&&!toETHide.contains(comp1)){
	//					String comp2 = ""; 
	//					if(cd.getParameterWithId(2)!= null){
	//						if(cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>").length>1){
	//							comp2 = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>")[1].split("</center></html>")[0];
	//						}else{
	//							comp2 = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1];			
	//						}
	//					}
	//					if(paramB!=null &&!toHide.contains(paramB)&&!toETHide.contains(comp2)){
	//						HashMap current = null;
	//						if(visibleCds.containsKey(paramA+";"+paramB)){
	//							current = (HashMap)visibleCds.get(paramA+";"+paramB);
	//						}else{
	//							current = new HashMap();
	//						}
	//						current.put(cd.getName(), cd);
	//						visibleCds.put(paramA+";"+paramB,current);
	//					}
	//					if(paramB ==null){
	//						HashMap current = null;
	//						if(visibleCds.containsKey(paramA)){
	//							current = (HashMap)visibleCds.get(paramA);
	//						}else{
	//							current = new HashMap();
	//						}
	//						current.put(cd.getName(), cd);
	//						visibleCds.put(paramA, current);
	//					}
	//				}
	//			}
	//
	//		}else{
	//			for(ConstraintDefinition cd: this.decViz.getModel().getModel().getConstraintDefinitions()){
	//				String paramA = "EMPTY_PARAM";
	//				String paramB = "EMPTY_PARAM";
	//				if(cd.getBranches(cd.getParameters().iterator().next()).iterator().hasNext()){
	//					paramA = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName();
	//				}
	//				if(cd.getParameterWithId(2)!= null){
	//					if(cd.getBranches(cd.getParameterWithId(2)).iterator().hasNext()){
	//						paramB = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>");
	//					}
	//				}
	//				String comp1 = "EMPTY_PARAM";
	//				if(cd.getBranches(cd.getParameters().iterator().next()).iterator().hasNext()){
	//					if(cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>").length>1){
	//						comp1 = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>")[1].split("</center></html>")[0];
	//					}else{
	//						comp1 = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1];			
	//					}
	//				}
	//				if(!toSkip.contains(cd.getName())&&!toHide.contains(paramA)&&!toETHide.contains(comp1)){
	//					String comp2 = "EMPTY_PARAM"; 
	//					if(cd.getParameterWithId(2)!= null){
	//						if(cd.getBranches(cd.getParameterWithId(2)).iterator().hasNext()){
	//							if(cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>").length>1){
	//								comp2 = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("</center><center>")[1].split("</center></html>")[0];
	//							}else{
	//								comp2 = cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-")[cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>").split("-").length-1];			
	//							}
	//						}
	//					}
	//					if(paramB!=null &&!toHide.contains(paramB)&&!toETHide.contains(comp2)){
	//						HashMap current = null;
	//						if(visibleCds.containsKey(paramA+";"+paramB)){
	//							current = (HashMap)visibleCds.get(paramA+";"+paramB);
	//						}else{
	//							current = new HashMap();
	//						}
	//						current.put(cd.getName(), cd);
	//						visibleCds.put(paramA+";"+paramB,current);
	//					}
	//					if(paramB ==null){
	//						HashMap current = null;
	//						if(visibleCds.containsKey(paramA)){
	//							current = (HashMap)visibleCds.get(paramA);
	//						}else{
	//							current = new HashMap();
	//						}
	//						current.put(cd.getName(), cd);
	//						visibleCds.put(paramA, current);
	//					}
	//				}
	//			}
	//
	//			for(ConstraintDefinition cd: this.decViz.getModel().getModel().getConstraintDefinitions()){
	//				String key = "EMPTY_PARAM";
	//				if(cd.getBranches(cd.getParameters().iterator().next()).iterator().hasNext()){
	//					key = cd.getBranches(cd.getParameters().iterator().next()).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>");
	//				}
	//				if(cd.getParameterWithId(2)!= null){
	//					if(cd.getBranches(cd.getParameters().iterator().next()).iterator().hasNext()){
	//					key = key + ";" + cd.getBranches(cd.getParameterWithId(2)).iterator().next().getName().replace("<html><body text=404040>","<html>").replace("</body></html>", "</html>");
	//					}else{
	//						key = key + ";EMPTY_PARAM";
	//					}
	//				}
	//				if(!(decViz.getVisiblesupportRule().get(cd.getId())>=minSupp)){
	//					if(visibleCds.containsKey(key)&&((HashMap)visibleCds.get(key)).containsKey(cd.getName())){
	//						((HashMap)visibleCds.get(key)).remove(cd.getName());
	//					}
	//				}else{
	//
	//					if(visibleCds.containsKey(key)){
	//						if(cd.getName().equals("chain response")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")){
	//								ConstraintDefinition cs = (ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession");
	//								if(decViz.getVisiblesupportRule().get(cs.getId())>=minSupp){
	//									((HashMap)visibleCds.get(key)).remove(cd.getName());
	//								}
	//							}
	//						}
	//
	//						if(cd.getName().equals("chain precedence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")){
	//								ConstraintDefinition cs = (ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession");
	//								if(decViz.getVisiblesupportRule().get(cs.getId())>=minSupp){
	//									((HashMap)visibleCds.get(key)).remove(cd.getName());
	//								}
	//							}
	//						}
	//
	//
	//
	//						if(cd.getName().equals("alternate response")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//
	//						if(cd.getName().equals("alternate precedence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//						if(cd.getName().equals("alternate succession")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//						if(cd.getName().equals("response")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//						if(cd.getName().equals("succession")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//						if(cd.getName().equals("precedence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("init")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("init")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//						if(cd.getName().equals("responded existence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//
	//							String reverseKey = key.split(";")[1]+";"+key.split(";")[0];
	//
	//							if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("precedence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey.split(";")[0])&&((HashMap)visibleCds.get(reverseKey.split(";")[0])).containsKey("init")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey.split(";")[0])).get("init")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//
	//						}
	//
	//
	//						if(cd.getName().equals("co-existence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(key)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//
	//
	//							String reverseKey = key.split(";")[1]+";"+key.split(";")[0];
	//
	//
	//							if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("chain response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("chain response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("precedence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("precedence")).getId())>=minSupp)&&((HashMap)visibleCds.get(reverseKey)).containsKey("alternate response")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("alternate response")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}	
	//						}
	//
	//
	//
	//						if(cd.getName().equals("existence")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("init")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("init")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("existence2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("existence2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("existence3")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("existence3")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly1")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly1")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//						if(cd.getName().equals("existence2")){
	//							if (((HashMap)visibleCds.get(key)).containsKey("existence3")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("existence3")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//						if(cd.getName().equals("absence2")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly1")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly1")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//						if(cd.getName().equals("absence3")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("absence2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("absence2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly1")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly1")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exactly2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exactly2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//						if(cd.getName().equals("choice")){
	//							if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("init")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("init")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("existence2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("existence2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("existence3")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("existence3")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("exactly1")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("exactly1")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("exactly2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("exactly2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("init")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("init")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("existence2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("existence2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("existence3")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("existence3")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("exactly1")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("exactly1")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("exactly2")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("exactly2")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(((HashMap)visibleCds.get(key)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//						if(cd.getName().equals("not chain succession")){
	//							if(((HashMap)visibleCds.get(key)).containsKey("not succession")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("not succession")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("not co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("not co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//							String reverseKey = key.split(";")[1]+";"+key.split(";")[0];
	//
	//							if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("not co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("not co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//
	//
	//
	//						if(cd.getName().equals("not succession")){
	//							if (((HashMap)visibleCds.get(key)).containsKey("not co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("not co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (((HashMap)visibleCds.get(key)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//
	//							String reverseKey = key.split(";")[1]+";"+key.split(";")[0];
	//
	//							if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("not co-existence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("not co-existence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//
	//
	//						if(cd.getName().equals("not co-existence")){
	//							if (((HashMap)visibleCds.get(key)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[0])&&((HashMap)visibleCds.get(key.split(";")[0])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[0])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}else if(visibleCds.containsKey(key.split(";")[1])&&((HashMap)visibleCds.get(key.split(";")[1])).containsKey("absence")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(key.split(";")[1])).get("absence")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//							String reverseKey = key.split(";")[1]+";"+key.split(";")[0];
	//
	//							if (visibleCds.containsKey(reverseKey)&&((HashMap)visibleCds.get(reverseKey)).containsKey("exclusive choice")&&(decViz.getVisiblesupportRule().get(((ConstraintDefinition)((HashMap)visibleCds.get(reverseKey)).get("exclusive choice")).getId())>=minSupp)){
	//								((HashMap)visibleCds.get(key)).remove(cd.getName());
	//							}
	//						}
	//
	//
	//					}
	//
	//
	//				}
	//			}
	//		}
	//
	//	
	//		return visibleCds;
	//	}


}